
/**
 * 检测网络返回的状态，正常返回业务数据则status为true，反之为false
 */
function _checkResponse(res, silent) {
    const curPage = getApp().globalData.curPage;

    const showToast = function(title) {
        if (!silent) {
            uni.$kit.showToast(title)
        }
    }
    
    // http请求返回判断
    if (res.hasOwnProperty("statusCode") && res.statusCode != 200) {
        showToast(res.statusCode ? curPage.$t(`http.status-${res.statusCode}`) : curPage.$t('common.unknown-error'))
        return false;
    }

    // ACM系统token过期时重新登录
    if (res.data && res.data.status != 200) {
        if (res.data.status == 40301) {
            if (!silent && !res.loginApi && curPage.mixin_toLogin) {
                setTimeout(_ => {
                    uni.$kit.showModal2(curPage.$t('common.to-login'),
                        curPage.$t('button.cancel'), 
                        curPage.$t('login.login'), 
                        _ => {
                        curPage.mixin_toLogin();
                    });
                }, 300)
            }
        } else {
            showToast(res.data.status ? curPage.$t(`http.status-${res.data.status}`) : curPage.$t('common.unknown-error'))
        }
        return false
    }
    
    // uniCloud调用云对象或云函数返回判断
    if (res.hasOwnProperty("errCode") && res.errCode) {
        showToast(res.errMsg)
        return false;
    }
    
    return true;
}

/**
 * 检测网络返回的状态，正常返回业务数据则status为true，反之为false
 */
function checkStatus(res, silent = true) {
    if (res instanceof Array) {
        for (let i = 0; i < res.length; i++) {
            if (!_checkResponse(res[i], silent)) {
                return false
            }
        }
        return true
    }
    return _checkResponse(res, silent)
}

export default {
    checkStatus
}
