/**
 * 文件的媒体类型
 */
const MIME_TYPES = new Map([
    //applications
    ["ai", "application/postscript"],
    ["eps", "application/postscript"],
    ["exe", "application/octet-stream"],
    ["doc", "application/vnd.ms-word"],
    ["docx", "application/vnd.ms-word"],
    ["xls", "application/vnd.ms-excel"],
    ["xlsx", "application/vnd.ms-excel"],
    ["ppt", "application/vnd.ms-powerpoint"],
    ["pptx", "application/vnd.ms-powerpoint"],
    ["pps", "application/vnd.ms-powerpoint"],
    ["pdf", "application/pdf"],
    ["xml", "application/xml"],
    ["odt", "application/vnd.oasis.opendocument.text"],
    ["swf", "application/x-shockwave-flash"],

    // archives
    ["gz", "application/x-gzip"],
    ["tgz", "application/x-gzip"],
    ["bz", "application/x-bzip2"],
    ["bz2", "application/x-bzip2"],
    ["tbz", "application/x-bzip2"],
    ["zip", "application/zip"],
    ["rar", "application/x-rar"],
    ["tar", "application/x-tar"],
    ["7z", "application/x-7z-compressed"],

    // texts
    ["txt", "text/plain"],
    ["php", "text/x-php"],
    ["html", "text/html"],
    ["htm", "text/html"],
    ["js", "text/javascript"],
    ["css", "text/css"],
    ["rtf", "text/rtf"],
    ["rtfd", "text/rtfd"],
    ["py", "text/x-python"],
    ["java", "text/x-java-source"],
    ["rb", "text/x-ruby"],
    ["sh", "text/x-shellscript"],
    ["pl", "text/x-perl"],
    ["sql", "text/x-sql"],

    // images
    ["bmp", "image/x-ms-bmp"],
    ["jpg", "image/jpeg"],
    ["jpeg", "image/jpeg"],
    ["gif", "image/gif"],
    ["png", "image/png"],
    ["tif", "image/tiff"],
    ["tiff", "image/tiff"],
    ["tga", "image/x-targa"],
    ["psd", "image/vnd.adobe.photoshop"],

    //audio
    ["mp3", "audio/mpeg"],
    ["mid", "audio/midi"],
    ["ogg", "audio/ogg"],
    ["mp4a", "audio/mp4"],
    ["wav", "audio/wav"],
    ["wma", "audio/x-ms-wma"],

    // video
    ["avi", "video/x-msvideo"],
    ["dv", "video/x-dv"],
    ["mp4", "video/mp4"],
    ["mpeg", "video/mpeg"],
    ["mpg", "video/mpeg"],
    ["mov", "video/quicktime"],
    ["wm", "video/x-ms-wmv"],
    ["flv", "video/x-flv"],
    ["mkv", "video/x-matroska"],

    ["unknown", "text/plain"]
]);

const TYPE_TO_COLOR = new Map([
    //applications
    ["ai", 'gray'],
    ["eps", 'gray'],
    ["exe", 'gray'],
    ["doc", 'blue'],
    ["docx", 'blue'],
    ["xls", 'green'],
    ["xlsx", 'green'],
    ["ppt", 'red'],
    ["pptx", 'red'],
    ["pps", 'gray'],
    ["pdf", 'orange'],
    ["xml", 'gray'],
    ["odt", 'gray'],
    ["swf", 'gray'],

    // archives
    ["gz", 'yellow'],
    ["tgz", 'yellow'],
    ["bz", 'yellow'],
    ["bz2", 'yellow'],
    ["tbz", 'yellow'],
    ["zip", 'yellow'],
    ["rar", 'yellow'],
    ["tar", 'yellow'],
    ["7z", 'yellow'],

    // texts
    ["txt", 'gray'],
    ["php", 'gray'],
    ["html", 'gray'],
    ["htm", 'gray'],
    ["js", 'gray'],
    ["css", 'gray'],
    ["rtf", 'gray'],
    ["rtfd", 'gray'],
    ["py", 'gray'],
    ["java", 'gray'],
    ["rb", 'gray'],
    ["sh", 'gray'],
    ["pl", 'gray'],
    ["sql", 'gray'],

    // images
    ["bmp", 'purple'],
    ["jpg", 'purple'],
    ["jpeg", 'purple'],
    ["gif", 'purple'],
    ["png", 'purple'],
    ["tif", 'purple'],
    ["tiff", 'purple'],
    ["tga", 'purple'],
    ["psd", 'purple'],

    //audio
    ["mp3", 'red'],
    ["mid", 'red'],
    ["ogg", 'red'],
    ["mp4a", 'red'],
    ["wav", 'red'],
    ["wma", 'red'],

    // video
    ["avi", 'olive'],
    ["dv", 'olive'],
    ["mp4", 'olive'],
    ["mpeg", 'olive'],
    ["mpg", 'olive'],
    ["mov", 'olive'],
    ["wm", 'olive'],
    ["flv", 'olive'],
    ["mkv", 'olive'],

    ["unknown", 'gray']
]);

const TYPE_TO_ICON = new Map([
    //applications
    ["ai", "custom-icon-ai-fill"],
    ["eps", "custom-icon-file-other-fill"],
    ["exe", "custom-icon-file-code"],
    ["doc", "custom-icon-word-fill"],
    ["docx", "custom-icon-word-fill"],
    ["xls", "custom-icon-xls-fill"],
    ["xlsx", "custom-icon-xls-fill"],
    ["ppt", "custom-icon-ppt-fill"],
    ["pptx", "custom-icon-ppt-fill"],
    ["pps", "custom-icon-file-other-fill"],
    ["pdf", "custom-icon-pdf-fill"],
    ["xml", "custom-icon-file-code"],
    ["odt", "custom-icon-file-other-fill"],
    ["swf", "custom-icon-file-code"],

    // archives
    ["gz", "custom-icon-zip-fill"],
    ["tgz", "custom-icon-zip-fill"],
    ["bz", "custom-icon-zip-fill"],
    ["bz2", "custom-icon-zip-fill"],
    ["tbz", "custom-icon-zip-fill"],
    ["zip", "custom-icon-zip-fill"],
    ["rar", "custom-icon-zip-fill"],
    ["tar", "custom-icon-zip-fill"],
    ["7z", "custom-icon-zip-fill"],

    // texts
    ["txt", "custom-icon-txt-fill"],
    ["php", "custom-icon-file-code"],
    ["html", "custom-icon-file-code"],
    ["htm", "custom-icon-file-code"],
    ["js", "custom-icon-file-code"],
    ["css", "custom-icon-file-code"],
    ["rtf", "custom-icon-file-code"],
    ["rtfd", "custom-icon-file-code"],
    ["py", "custom-icon-file-code"],
    ["java", "custom-icon-file-code"],
    ["rb", "custom-icon-file-code"],
    ["sh", "custom-icon-file-code"],
    ["pl", "custom-icon-file-code"],
    ["sql", "custom-icon-file-code"],

    // images
    ["bmp", "custom-icon-image-fill"],
    ["jpg", "custom-icon-image-fill"],
    ["jpeg", "custom-icon-image-fill"],
    ["gif", "custom-icon-image-fill"],
    ["png", "custom-icon-image-fill"],
    ["tif", "custom-icon-image-fill"],
    ["tiff", "custom-icon-image-fill"],
    ["tga", "custom-icon-image-fill"],
    ["psd", "custom-icon-image-fill"],

    //audio
    ["mp3", "custom-icon-music-fill"],
    ["mid", "custom-icon-music-fill"],
    ["ogg", "custom-icon-music-fill"],
    ["mp4a", "custom-icon-music-fill"],
    ["wav", "custom-icon-music-fill"],
    ["wma", "custom-icon-music-fill"],

    // video
    ["avi", "custom-icon-vedio-fill"],
    ["dv", "custom-icon-vedio-fill"],
    ["mp4", "custom-icon-vedio-fill"],
    ["mpeg", "custom-icon-vedio-fill"],
    ["mpg", "custom-icon-vedio-fill"],
    ["mov", "custom-icon-vedio-fill"],
    ["wm", "custom-icon-vedio-fill"],
    ["flv", "custom-icon-vedio-fill"],
    ["mkv", "custom-icon-vedio-fill"],

    ["unknown", 'custom-icon-file-other-fill']
]);

/**
 * 支持的可预览类型
 */
const SUPPORT_PREVIEW_TYPES = [
    'html', 
    'txt', 
    'jpg', 
    'jpeg', 
    'gif', 
    'png', 
    'art', 
    'au', 
    'aiff', 
    'xbm', 
    'pdf', 
    'doc', 
    'docx', 
    'rtf', 
    'xls', 
    'xlsx', 
    'csv',
]

/**
 * 获取文件的媒体类型
 */
String.prototype.getFileMimeType = function () {
    return MIME_TYPES.get(this.getFileType()) || "text/plain";
}

/**
 * 获取文件类型对应的颜色
 */
String.prototype.getFileColor = function() {
    return TYPE_TO_COLOR.get(this.getFileType()) || 'gray';
}

/**
 * 获取文件类型对应的图标
 */
String.prototype.getFileIcon = function () {
    return TYPE_TO_ICON.get(this.getFileType()) || 'custom-icon-file-other-fill';
}

/**
 * 可预览的类型
 */
String.prototype.isSupportType = function () {
    if (this.startsWith(".")) {
        return SUPPORT_PREVIEW_TYPES.includes(this.substr(1));
    }
    return SUPPORT_PREVIEW_TYPES.includes(this);
}

/**
 * 判断文件是否为图片类型
 * @param {Object} file
 */
function isImageFile(file) {
    return file.type.getFileMimeType().startsWith('image/')
}

/**
 * 判断files是否全部为图片文件
 * @param {Object} files
 */
function isAllImageFile(files) {
    for (let file of files) {
        if (!isImageFile(file)) {
            return false
        }
    }
    return true
}

/**
 * 判断文件是否为pdf类型
 * @param {Object} file
 */
function isPdfFile(file) {
    return file.type.getFileMimeType().endsWith('/pdf')
}

/**
 * 判断文件是否为excel类型
 * @param {Object} file
 */
function isExcelFile(file) {
    return file.type.getFileMimeType().endsWith('ms-excel')
}

/**
 * 判断文件是否为可支持预览的类型
 * @param {Object} file
 */
function canPreviewFile(file) {
    return isImageFile(file) || isExcelWordFile(file) || isPdfFile(file) ||
        file.type.getFileMimeType().endsWith('text/')
}

/**
 * 判断文件是否为excel或doc类型
 * @param {Object} doc
 */
function isExcelWordFile(doc) {
    const mimeType = doc.type.getFileMimeType()
    return mimeType.endsWith('ms-word') || mimeType.endsWith('ms-excel')
}

// #ifdef H5
/**
 * 通过文件对象获取本地文件的路径
 * @param {Object} file
 */
function getLocalFileURL(file) {
    var getUrl = null;
    if (window.createObjectURL != undefined) { // basic
        getUrl = window.createObjectURL(file);
    } else if (window.URL != undefined) { // mozilla(firefox)
        getUrl = window.URL.createObjectURL(file);
    } else if (window.webkitURL != undefined) { // webkit or chrome
        getUrl = window.webkitURL.createObjectURL(file);
    }
    return getUrl;
}
// #endif


/**
 * 计算压缩比
 * @param {Object} bytes
 */
function compressRate(bytes) {
    const m = bytes / (1024 * 1024)
    if (m < 2) {
        return 100
    } else {
        return Math.floor(200 / m)
    }
}

/**
 * 构建文件访问路径
 * @param {Number} fileId
 */
function buildPreviewUrl(fileId, thumbnail = false) {
    if (!fileId) {
        return "";
    }
    
    // 当fileId参数为网络文件路径时直接返回该路径
    if (uni.$kit.isString(fileId) && (fileId.startsWith("http://") || fileId.startsWith("https://"))) {
        return fileId;
    }
    
    const token = uni.getStorageSync('uni_id_token');
    if (thumbnail) {
        return uni.$http.url.baseUrl() + `/doc/file/online/thumbnail/view/ext/api?fileId=${fileId}&token=${token}`;
    }
    return uni.$http.url.baseUrl() + `/doc/file/online/view/ext/api?fileId=${fileId}&token=${token}`;
}

export default {
    isImageFile,
    isAllImageFile,
    isPdfFile,
    isExcelWordFile,
    isExcelFile,
    canPreviewFile,
    // #ifdef H5
    getLocalFileURL,
    // #endif
    compressRate,
    buildPreviewUrl
};
