import type from "../type.js"

/**
 * 求和
 * @param {Object} handleItem 返回求和字段
 * @param {Object} parseFunc 求和字段为字符串时，解析转换方法
 * @param {Object} mathFunc 求和字段为数字时，取值数学方法
 */
function sum(handleItem, parseFunc, mathFunc) {
    let sum = 0;
    for (let i = 0; i < this.length; i++) {
        let item = this[i];
        if (item) {
            if (type.isObject(item)) {
                item = handleItem ? handleItem(item): item.value;
            }
            
            if (type.isNumber(item)) {
                sum += mathFunc ? mathFunc(item): item;
            } else if (type.isString(item)) {
                if (parseFunc) {
                    sum += parseFunc(item);
                } else {
                    sum += item.includes(".") ? parseFloat(item): parseInt(item);
                }
            }
        }
    }
    return sum;
}

/**
 * 整数数组求和
 */
Array.prototype.sumInt = function (handler, mathFunc = "floor") {
    return sum.call(this, handler, item => parseInt(item), item => Math[mathFunc](item));
}

/**
 * 浮点数数组求和
 */
Array.prototype.sumFloat = function (handler) {
    return sum.call(this, handler, item => parseFloat(item));
}

/**
 * 数组求和
 */
Array.prototype.sum = function (handler) {
    return sum.call(this, handler);
}

/**
 * 数组去重
 */
Array.prototype.unique = function (key = 'id') {
    const tmpList = [];
    for (let i = 0; i < this.length; i++) {
        const item = this[i];
        if (!tmpList.find(tmpItem => tmpItem[key] == item[key])) {
            tmpList.push(item);
        }
    }
    return tmpList;
}

export default {}