/*----------------------------格式化字符串定义---------------------------*/

// 格式化字符串
Date.FMT_YEAR = "yyyy";
Date.FMT_MONTH = "M";
Date.FMT_DAY = "d";

Date.FMT_DATE = "yyyy-MM-dd";
Date.FMT_DATE_SHORT = "yyyy-MM";
Date.FMT_DATE_SAME_YEAR = "MM-dd";
Date.FMT_DATE_SLASH = "yyyy/MM/dd";

Date.FMT_TIME_LONG = "HH:mm:ss.S";
Date.FMT_TIME = "HH:mm:ss";
Date.FMT_TIME_SHORT = "HH:mm";

Date.FMT_DATE_TIME_LONG = "yyyy-MM-dd HH:mm:ss.S";
Date.FMT_DATE_TIME = "yyyy-MM-dd HH:mm:ss";
Date.FMT_DATE_TIME_SHORT = "yyyy-MM-dd HH:mm";

// 汉字
Date.FMT_YEAR_ZH = "yyyy年";
Date.FMT_MONTH_ZH = "M月";
Date.FMT_DAY_ZH = "d日";

Date.FMT_DATE_ZH = "yyyy年M月d日";
Date.FMT_DATE_SHORT_ZH = "yyyy年M月";
Date.FMT_DATE_SAME_YEAR_ZH = "M月d日";

Date.FMT_TIME_LONG_ZH = "HH时:mm分:ss秒.S";
Date.FMT_TIME_ZH = "HH时:mm分:ss秒";
Date.FMT_TIME_SHORT_ZH = "HH时:mm分";

Date.FMT_DATE_TIME_LONG_ZH = "yyyy年M月d日 HH时:mm分:ss秒.S";
Date.FMT_DATE_TIME_ZH = "yyyy年M月d日 HH时:mm分:ss秒";
Date.FMT_DATE_TIME_SHORT_ZH = "yyyy年M月d日 HH时:mm分";

// 点
Date.FMT_DATE_POINT = "yyyy.M.d";
Date.FMT_DATE_SHORT_POINT = "yyyy.M";
Date.FMT_DATE_SAME_YEAR_POINT = "M.d";
Date.FMT_DATE_TIME_SHORT_POINT = "yyyy.M.d HH:mm";

// 英文
Date.FMT_DATE_EN = "MM/dd/yyyy";
Date.FMT_DATE_SHORT_EN = "MM/yyyy";
Date.FMT_DATE_SAME_YEAR_EN = "M/d";
Date.FMT_DATE_POINT_EN = "M.d.yyyy";
Date.FMT_DATE_SHORT_POINT_EN = "M.yyyy";

// 一周周号，日历标准顺序
Date.WEEK_DAYS = ["日", "一", "二", "三", "四", "五", "六"];
Date.WEEK_DAYS_EN = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];

/**
 * 日期格式化
 */
Date.prototype.format = function(fmt) {
    let o = {
        "M+": this.getMonth() + 1, //月份
        "d+": this.getDate(), //日
        "h+": this.getHours() % 12 == 0 ? 12 : this.getHours() % 12, //小时
        "H+": this.getHours(), //小时
        "m+": this.getMinutes(), //分
        "s+": this.getSeconds(), //秒
        "q+": Math.floor((this.getMonth() + 3) / 3), //季度
        "S": this.getMilliseconds() //毫秒
    };
    let week = {
        "0": "/u65e5",
        "1": "/u4e00",
        "2": "/u4e8c",
        "3": "/u4e09",
        "4": "/u56db",
        "5": "/u4e94",
        "6": "/u516d"
    };

    if (/(y+)/.test(fmt)) {
        fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
    }

    if (/(E+)/.test(fmt)) {
        const tmp = RegExp.$1.length > 1 ? RegExp.$1.length > 2 ? "/u661f/u671f" : "/u5468" : "";
        fmt = fmt.replace(RegExp.$1, tmp + week[this.getDay() + ""]);
    }

    for (let k in o) {
        if (new RegExp("(" + k + ")").test(fmt)) {
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ? (o[k]) : (("00" + o[k]).substr(("" + o[k])
                .length)));
        }
    }
    return fmt;
};

/**
 * 获取日期格式字符串
 * @param {Boolean} zh 中文
 * @param {Boolean} point 分隔符-点
 * @param {Boolean} short 短日期
 * @param {Boolean} sameYear 同年
 */
Date._i18nDateFormat = function (
    zh = false, 
    point = false, 
    short = false, 
    sameYear = false) {
    if (uni.isCn()) {
        if (point) {
            return short ? Date.FMT_DATE_SHORT_POINT: sameYear ? Date.FMT_DATE_SAME_YEAR_POINT: Date.FMT_DATE_POINT
        }
        if (zh) {
            return short ? Date.FMT_DATE_SHORT_ZH: sameYear ? Date.FMT_DATE_SAME_YEAR_ZH: Date.FMT_DATE_ZH
        }
        return short ? Date.FMT_DATE_SHORT: sameYear ? Date.FMT_DATE_SAME_YEAR: Date.FMT_DATE
    } else {
        if (point) {
            return short ? Date.FMT_DATE_SHORT_POINT_EN: sameYear ? Date.FMT_DATE_SAME_YEAR_POINT: Date.FMT_DATE_POINT_EN
        }
        return short ? Date.FMT_DATE_SHORT_EN: sameYear ? Date.FMT_DATE_SAME_YEAR: Date.FMT_DATE_EN;
    }
}

/**
 * 获取日期时间格式字符串
 */
Date._i18nDateTimeFormat = function (
    zh = false, 
    point = false, 
    dateShort = false, 
    sameYear = false, 
    timeShort = true,
    timeZh = false) {
    const dateFormat = Date._i18nDateFormat(zh, point, dateShort, sameYear)
    let timeFormat = timeShort ? Date.FMT_TIME_SHORT: Date.FMT_TIME
    if (timeZh) {
        timeFormat = timeShort ? Date.FMT_TIME_SHORT_ZH: Date.FMT_TIME_ZH
    }
    return `${dateFormat} ${timeFormat}`
}

/**
 * 获取今天日期字符串
 */
Date.fmtTodayText = (fmt) => {
    const dateFormat = fmt || Date._i18nDateFormat();
    return new Date().format(dateFormat);
}

/**
 * 获取今天日期字符串
 */
Date.todayText = (zh = false, point = false, short = false) => {
    return Date.fmtTodayText(Date._i18nDateFormat(zh, point, short, false));
}

/**
 * 获取今天日期字符串
 */
Date.todayCNText = (short = false) => {
    return Date.todayText(true, false, short);
}

/**
 * 获取今天日期字符串
 */
Date.todayPointText = (short = false) => {
    return Date.todayText(false, true, short);
}

/**
 * 获取今天日期字符串
 */
Date.todayTimeText = (zh = false, point = false, timeShort = false) => {
    return Date.fmtTodayText(Date._i18nDateTimeFormat(zh, point, false, false, timeShort, zh));
}

/**
 * 获取今天日期字符串
 */
Date.todayTimeCNText = (timeShort = true) => {
    return Date.todayTimeText(true, false, timeShort);
}

/**
 * 获取今天日期字符串
 */
Date.todayTimePointText = (timeShort = true) => {
    return Date.todayTimeText(false, true, timeShort);
}

/**
 * 获取昨天日期
 */
Date.fmtYestodayText = (fmt) => {
    const dateFormat = fmt || Date._i18nDateFormat();
    const timestamp = new Date().getTime() - 24 * 60 * 60 * 1000;
    return new Date(timestamp).format(dateFormat);
}

/**
 * 获取昨天日期
 */
Date.yestodayText = (zh = false, point = false) => {
    return Date.fmtYestodayText(Date._i18nDateFormat(zh, point, false, false));
}

/**
 * 获取昨天日期字符串
 */
Date.yestodayCNText = () => {
    return Date.yestodayText(true);
}

/**
 * 获取昨天日期字符串
 */
Date.yestodayPointText = () => {
    return Date.yestodayText(false, true);
}

/**
 * 获取昨天日期
 */
Date.yestodayTimeText = (zh = false, point = false, timeShort = false) => {
    return Date.fmtYestodayText(Date._i18nDateTimeFormat(zh, point, false, false, timeShort, zh));
}

/**
 * 获取昨天日期字符串
 */
Date.yestodayTimeCNText = (timeShort = true) => {
    return Date.yestodayTimeText(true, false, timeShort);
}

/**
 * 获取昨天日期字符串
 */
Date.yestodayTimePointText = (timeShort = true) => {
    return Date.yestodayTimeText(false, true, timeShort);
}

/**
 * 获取明天日期
 */
Date.fmtTomorrowText = (fmt) => {
    const dateFormat = fmt || Date._i18nDateFormat();
    const timestamp = new Date().getTime() + 24 * 60 * 60 * 1000;
    return new Date(timestamp).format(dateFormat);
}

/**
 * 获取明天日期
 */
Date.tomorrowText = (zh = false, point = false) => {
    return Date.fmtYestodayText(Date._i18nDateFormat(zh, point, false, false));
}

/**
 * 获取明天日期字符串
 */
Date.tomorrowCNText = () => {
    return Date.tomorrowText(true);
}

/**
 * 获取明天日期字符串
 */
Date.tomorrowPointText = () => {
    return Date.tomorrowText(false, true);
}

/**
 * 获取明天日期
 */
Date.tomorrowTimeText = (zh = false, point = false, timeShort = false) => {
    return Date.fmtTomorrowText(Date._i18nDateTimeFormat(zh, point, false, false, timeShort, zh));
}

/**
 * 获取明天日期字符串
 */
Date.tomorrowTimeCNText = (timeShort = true) => {
    return Date.tomorrowTimeText(true, false, timeShort);
}

/**
 * 获取明天日期字符串
 */
Date.tomorrowTimePointText = (timeShort = true) => {
    return Date.tomorrowTimeText(false, true, timeShort);
}

/**
 * 获取上个月的今天或者下个月的今天
 * 如果上个月或者下个月没有今天，就取上个月或者下个月的最后一天。
 * 传参数 1是下个月的今天，-1是取上个月的今天。
 * @param {Number} type
 * @param {Number|String} date
 */
Date.getXmonthToday = function (type, date) {
    // type 0 是当天 -1 是上个月   1是下个月
    let now = date ? new Date(date): new Date(); // 可以传值调式 now = new Date(2019,2,30); 今天是3月30号
    let year = now.getFullYear(); //getYear()+1900=getFullYear()
    let month = now.getMonth() + 1; //0-11表示1-12月
    let day = now.getDate(); // 当天日期
    if (parseInt(month) < 10) {
        month = "0" + month;
    }
    if (parseInt(day) < 10) {
        day = "0" + day;
    }
    now = year + '-' + month + '-' + day; // 如果取当月日期可直接 return 返回

    let preMonth = parseInt(month) - 1;
    preMonth = preMonth < 10 ? '0' + preMonth : preMonth; // 获取上个月的值
    let nextMonth = parseInt(month) + 1;
    nextMonth = nextMonth < 10 ? '0' + nextMonth : nextMonth; // 获取下个月个月的值

    if (parseInt(month) == 1 && type == -1) { //如果是1月份，要上个月 ，则取上一年的12月份
        return (parseInt(year) - 1) + '-12-' + day;
    } else if (parseInt(month) == 12 && type == 1) { // 如果是12月，要下个月，取下一年的1月
        return (parseInt(year) + 1) + '-01-' + day;
    }

    let preSize = new Date(year, parseInt(month) - 1, 0).getDate(); //上月总天数
    let nextSize = new Date(year, parseInt(month) + 1, 0).getDate(); //下月总天数

    if (preSize < parseInt(day) && type == -1) { // 取上个月，如果上个月总天数小于本月今天，取上个月最后一天    
        return year + '-' + preMonth + '-' + preSize;
    } else if (nextSize < parseInt(day) && type == 1) { // 如果下个月总天数小于本月今天，取下个月最后一天  
        return year + '-' + nextMonth + '-' + nextSize;
    }

    if (type == -1) {
        return year + '-' + preMonth + '-' + day;
    } else if (type == 1) {
        return year + '-' + nextMonth + '-' + day;
    } else if (type == 0) {
        return now;
    }
}

/**
 * 获取开始和结束日期间的所有日期
 * @param {Object} startDay
 * @param {Object} endDay
 */
Date.getDatesBetween = function (startDay, endDay) {
    let arr = [];
    let dates = [];

    // 设置两个日期UTC时间
    let db = new Date(startDay);
    let de = new Date(endDay);

    // 获取两个日期GTM时间
    let s = db.getTime() - 24 * 60 * 60 * 1000;
    let d = de.getTime() - 24 * 60 * 60 * 1000;

    // 获取到两个日期之间的每一天的毫秒数
    for (let i = s; i <= d;) {
        i = i + 24 * 60 * 60 * 1000;
        arr.push(parseInt(i))
    }

    // 获取每一天的时间  YY-MM-DD
    for (let j in arr) {
        let time = new Date(arr[j]);
        let year = time.getFullYear(time);
        let mouth = (time.getMonth() + 1) >= 10 ? (time.getMonth() + 1) : ('0' + (time.getMonth() + 1));
        let day = time.getDate() >= 10 ? time.getDate() : ('0' + time.getDate());
        let YYMMDD = year + '-' + mouth + '-' + day;
        dates.push(YYMMDD)
    }

    return dates
}

/**
 * 获取某年某季度起止日期, 如:["2020-01-01","2020-03-31"]
 */
Date.getQuarterDates = function (year, quarter) {
    const qs = new Date(year, (quarter - 1) * 3, 1);
    const qe = new Date(year, quarter * 3, 0);
    const startMonth = qs.getMonth() + 1;
    const endMonth = qe.getMonth() + 1;
    const startMonthText = startMonth < 10 ? "0" + startMonth : startMonth;
    const startDayText = qs.getDate() < 10 ? "0" + qs.getDate() : qs.getDate();
    const endMonthText = endMonth < 10 ? "0" + endMonth : endMonth;
    const endDayText = qe.getDate() < 10 ? "0" + qe.getDate() : qe.getDate();
    return [
        `${qs.getFullYear()}-${startMonthText}-${startDayText}`,
        `${qs.getFullYear()}-${endMonthText}-${endDayText}`
    ]
}

/**
 * 获取某年份一共有多少周
 */
Date.weekAmount = function (year) {
    let d = new Date(year, 0, 1);
    while (d.getDay() != 1) {
        d.setDate(d.getDate() + 1);
    }
    let to = new Date(year + 1, 0, 1);
    let i = 1;
    for (let from = d; from < to;) {
        from.setDate(from.getDate() + 6);
        if (from < to) {
            from.setDate(from.getDate() + 1);
        } else {
            to.setDate(to.getDate() - 1);
        }
        i++;
    }
    return i;
}

/**
 * 获取某年份周日期段和对应
 */
Date.weekNumToPeriod = function (year) {
    let d = new Date(year, 0, 1);
    while (d.getDay() != 1) {
        d.setDate(d.getDate() + 1);
    }
    let to = new Date(year + 1, 0, 1);
    const res = [];
    let i = 1;
    for (let from = d; from < to;) {
        const item = {
            week: i,
            start: `${year}-` + (from.getMonth() + 1 < 10 ? "0" + (from.getMonth() + 1) : from.getMonth() +
                    1) +
                "-" + (from.getDate() < 10 ? "0" + from.getDate() : from.getDate())
        }
        from.setDate(from.getDate() + 6);
        if (from < to) {
            item.end = `${year}-` + (from.getMonth() + 1 < 10 ? "0" + (from.getMonth() + 1) : from.getMonth() +
                    1) +
                "-" + (from.getDate() < 10 ? "0" + from.getDate() : from.getDate());
            from.setDate(from.getDate() + 1);
        } else {
            to.setDate(to.getDate() - 1);
            item.end = `${year}-` + (from.getMonth() + 1 < 10 ? "0" + (from.getMonth() + 1) : from.getMonth() +
                    1) +
                "-" + (from.getDate() < 10 ? "0" + from.getDate() : from.getDate());
        }
        res.push(item);
        i++;
    }
    return res;
}

export default {}
