import type from "../type.js"

/*----------------------金额格式化-----------------------*/
/**
 * 格式化金额
 */
String.prototype.moneyText = function (n = 2) {
    let s = this;
    if (!s) {
        s = 0
    }
    
    n = n >= 0 && n <= 20 ? n : 2;
    s = parseFloat((s + "").replace(/[^\d\.-]/g, "")).toFixed(n) + "";
    let l = s.split(".")[0].split("").reverse(),
        r = s.split(".")[1];
    let t = "";
    for (let i = 0; i < l.length; i++) {
        t += l[i] + ((i + 1) % 3 === 0 && (i + 1) !== l.length ? " ," : "");
    }
    return t.split("").reverse().join("") + "." + r;
}

/**
 * 恢复格式化后的金额字符为数字
 */
String.prototype.moneyNumber = function () {
    return parseFloat(this.replace(/[^\d\.-]/g, ""));
}

/*----------------------日期格式化-----------------------*/

/**
 * 格式化日期
 */
String.prototype.fmtDateText = function (fmt) {
    const dateFormat = fmt || Date._i18nDateFormat();
    let s = this;
    if (isNaN(this)) { // 日期字符串
        s = this.replace(/-/g, '/');
    } else {
        s = parseInt(this); // 时间戳字符串
    }
    return this ? new Date(s).format(dateFormat): ""
}

/**
 * 格式化日期
 */
String.prototype.dateText = function (zh = false, point = false, yearMonth = false) {
    const fmt = Date._i18nDateFormat(zh, point, yearMonth, false)
    return this.fmtDateText(fmt);
}

/**
 * 中文格式化日期
 */
String.prototype.dateCNText = function (yearMonth = false) {
    return this.dateText(true, false, yearMonth)
}

/**
 * 中文格式化年月
 */
String.prototype.yearMonthCNText = function () {
    return this.dateCNText(true, false, true)
}

/**
 * 点分隔符格式化日期
 */
String.prototype.datePointText = function (yearMonth = false) {
    return this.dateText(false, true, yearMonth)
}

/**
 * 日期和时间格式化日期
 */
String.prototype.dateTimeText = function (zh = false, point = false, timeShort = false) {
    const fmt = Date._i18nDateTimeFormat(zh, point, false, timeShort, zh)
    return this.fmtDateText(fmt);
}

/**
 * 中文格式化日期
 */
String.prototype.dateTimeCNText = function (timeShort = false) {
    return this.dateTimeText(true, false, false, timeShort, true);
}

/**
 * 点分隔符格式化日期
 */
String.prototype.dateTimePointText = function (yearMonth = false, timeShort = false) {
    return this.dateTimeText(false, true, yearMonth, timeShort, false);
}

/**
 * 将日期（时间戳）格式化为微信日期字符串
 */
String.prototype.chatText = function (yearMonth = false, point = false) {
    if (!this) {
        return ''
    }

    let result = "";
    let now = new Date();
    let dateStr = this.replace(/-/g, '/')
    let target = new Date(dateStr);

    if (now.getFullYear() === target.getFullYear()) { // 同年
        if (now.getMonth() === target.getMonth()) { // 同月
            switch (now.getDay() - target.getDay()) {
                case 0:
                    result = dateStr.fmtDateText(Date.FMT_TIME_SHORT);
                    break;
                case 1:
                    const yesterday = getApp().globalData.curPage.$t('component.yesterday')
                    result = `${yesterday} ` + (yearMonth ? "": dateStr.fmtDateText(Date.FMT_TIME_SHORT));
                    break;
                default:
                    result = dateStr.fmtDateText(Date._i18nDateTimeFormat(true, point, false, true, true));
                    if (yearMonth) {
                        result = dateStr.fmtDateText(Date._i18nDateFormat(true, point, false, true));
                    }
                    break;
            }
        } else {
            result = dateStr.fmtDateText(Date._i18nDateTimeFormat(true, point, false, true, true));
            if (yearMonth) {
                result = dateStr.fmtDateText(Date._i18nDateFormat(true, point, false, true));
            }
        }
    } else {
        result = dateStr.fmtDateText(Date._i18nDateTimeFormat(true, point));
        if (yearMonth) {
            result = dateStr.fmtDateText(Date._i18nDateFormat(true, point));
        }
    }
    return result;
}

/**
 * 将日期字符串转换为时间戳
 */
String.prototype.toTimestamp = function () {
    return new Date(this.replace(/-/g, '/')).getTime()
}

/**
 * 获取两个时间戳之间的天数
 * @param {Object} before 时间戳
 * @param {Boolen} round 去除时分秒
 * @returns {number}
 */
String.prototype.subDays = function (before, round = true) {
    if (!before) {
        before = round ? Date.todayText().toTimestamp(): new Date().getTime();
    }
    let beforeTime = before;
    if (!type.isNumber(before)) {
        if (!isNaN(before)) { // 时间戳字符串
            beforeTime = round ? before.dateText().toTimestamp(): parseInt(before);
        } else if (type.isString(before)) { // 日期字符串
            beforeTime = round ? before.dateText().toTimestamp(): before.toTimestamp();
        } else {
            beforeTime = round ? Date.todayText().toTimestamp(): new Date().getTime();
        }
    }
    let afterTime;
    if (!isNaN(this)) { // 时间戳字符串
        afterTime = round ? this.dateText().toTimestamp(): parseInt(this);
    } else { // 日期字符串
        afterTime = round ? this.dateText().toTimestamp(): this.toTimestamp();
    }
    return Math.floor((afterTime - beforeTime) / (24 * 3600 * 1000));
}

/**
 * 是否过期
 */
String.prototype.outOfDate = function (round = true) {
    return this.subDays("", round) < 0
}

/**
 * 获取上个月的今天
 */
String.prototype.prevMonthToday = function () {
    return Date.getXmonthToday(-1, this);
}

/**
 * 获取下个月的今天
 */
String.prototype.nextMonthToday = function () {
    return Date.getXmonthToday(1, this);
}

/**
 * 计算出指定日期为当前周周几
 */
String.prototype.weekNumber = function () {
    let myDate = new Date(this);
    if (uni.isCn()) {
        return Date.WEEK_DAYS[myDate.getDay()]
    } else {
        return Date.WEEK_DAYS_EN[myDate.getDay()]
    }
};

/**
 * 计算出指定日期所在周的所有日期(注意: 周日~周六)
 * 如: [12, 13, ...]
 */
String.prototype.daysOfWeek = function () {
    const curDate = new Date(this);
    const curDay = curDate.getDay();
    const curTime = curDate.getTime();
    return [0, 1, 2, 3, 4, 5, 6].map(i => new Date(curTime + 24 * 60 * 60 * 1000 * (i - curDay)).getDate())
}

/**
 * 计算出指定日期所在周的所有字符串格式日期(注意: 周日~周一)
 * 如:["2020-02-13",....]
 */
String.prototype.datesOfWeek = function (fromMonday = false) {
    let curDate = new Date(this);
    if (fromMonday && curDate.getDay() == 0) {
        curDate = new Date(this.beforeDayText());
    }
    const curDay = curDate.getDay();
    const curTime = curDate.getTime();
    return [0, 1, 2, 3, 4, 5, 6].map(i => {
        const time = curTime + 24 * 60 * 60 * 1000 * (i - curDay + (fromMonday ? 1: 0));
        return `${time}`.dateText();
    })
}

/**
 * 获取当前日期月份的所有日期, ['2023-05-01', ...]
 */
String.prototype.datesOfMonth = function () {
    const formatDate = function(pdate) {
        const newDate = new Date(pdate);
        let myyear = newDate.getFullYear();
        let mymonth = newDate.getMonth() + 1;
        let myweekday = newDate.getDate();
        mymonth < 10 ? mymonth = "0" + mymonth : mymonth;
        myweekday < 10 ? myweekday = "0" + myweekday : myweekday;
        return `${myyear}-${mymonth}-${myweekday}`;
    }

    const getMonthDaysCount = function(pdate) {
        const newDate = pdate ? new Date(pdate) : new Date();
        return new Date(newDate.getFullYear(), newDate.getMonth() + 1, 0).getDate();
    }

    const curDate = new Date(this);
    const num = getMonthDaysCount(this);
    const curMonth = [];
    for (let i = 1; i <= num; i++) {
        curMonth.push(formatDate(curDate.setDate(i)));
    }
    return curMonth
}

/**
 * 获取当前日期月份的所有日期, [1, 2, ...]
 */
String.prototype.daysOfMonth = function () {
    const getMonthDaysCount = function(pdate) {
        const newDate = pdate ? new Date(pdate) : new Date();
        return new Date(newDate.getFullYear(), newDate.getMonth() + 1, 0).getDate();
    }

    const curDate = new Date(this);
    const num = getMonthDaysCount(this);
    const curMonth = [];
    for (let i = 1; i <= num; i++) {
        curMonth.push(i);
    }
    return curMonth
}

/**
 * 获取指定日期前一天，如：2020-02-13
 */
String.prototype.beforeDayText = function () {
    const beforeDayTime = new Date(this).getTime() - 24 * 60 * 60 * 1000;
    return `${beforeDayTime}`.dateText();
}

/**
 * 获取指定日期后一天，如：2020-02-13
 */
String.prototype.afterDayText = function () {
    const afterTime = new Date(this).getTime() + 24 * 60 * 60 * 1000;
    return `${afterTime}`.dateText();
}

/**
 * 获取指定日期的前一周时间，如:["2020-02-13",....]
 * @param {Boolean} fromMonday [false: 周日~周一, true: 周一~周日] 
 */
String.prototype.datesOfBeforeWeek = function (fromMonday = false) {
    const beforeDay = this.datesOfWeek(true)[0].beforeDayText();
    if (fromMonday) {
        return beforeDay.datesOfWeek(fromMonday);
    } else {
        return beforeDay.beforeDayText().datesOfWeek(fromMonday);
    }
}

/**
 * 获取指定日期的后一周时间，如:["2020-02-13",....]
 * @param {Boolean} fromMonday [false: 周日~周一, true: 周一~周日] 
 */
String.prototype.datesOfAfterWeek = function (fromMonday = false) {
    const curDates = this.datesOfWeek(true);
    return curDates[curDates.length-1].afterDayText().datesOfWeek(fromMonday)
}

/**
 * 获取指定日期的前一月时间, 如:["2020-02-01",....]
 */
String.prototype.datesBeforeMonth = function () {
    return this.datesOfMonth()[0].beforeDayText().datesOfMonth();
}

/**
 * 获取指定日期的后一月时间, 如:["2020-02-01",....]
 */
String.prototype.datesAfterMonth = function () {
    const curDates = this.datesOfMonth();
    return curDates[curDates.length-1].afterDayText().datesOfMonth();
}

/**
 * 获取某年份一共有多少周
 */
String.prototype.weekAmount = function () {
    let year;
    if (isNaN(this)) {
        year = parseInt(this.fmtDateText(Date.FMT_YEAR));
    } else {
        year = parseInt(this);
    }
    return Date.weekAmount(year);
}

/**
 * 获取某年份周日期段和对应
 */
String.prototype.weekNumToPeriod = function () {
    let year;
    if (isNaN(this)) {
        year = parseInt(this.fmtDateText(Date.FMT_YEAR));
    } else {
        year = parseInt(this);
    }
    return Date.weekNumToPeriod(year);
}

/**
 * 获取指定日期当前周数
 */
String.prototype.weekCount = function () {
    //兼容ios，日期小于10前面补0 == 2021-09-08
    let year = ''
    let month = ''
    let day = ''
    const dateStr = this;
    if (dateStr) {
        year = dateStr.split('-')[0]
        if (dateStr.split('-')[1].substr(0, 1) == '0') {
            month = dateStr.split('-')[1]
        } else {
            month = dateStr.split('-')[1] < 10 ? '0' + dateStr.split('-')[1] : dateStr.split('-')[1]
        }

        if (dateStr.split('-')[2].substr(0, 1) == '0') {
            day = dateStr.split('-')[2]
        } else {
            day = dateStr.split('-')[2] < 10 ? '0' + dateStr.split('-')[2] : dateStr.split('-')[2]
        }
    }
    let date1 = dateStr ? new Date(`${year}-${month}-${day}`) : new Date();
    let date2 = dateStr ? new Date(`${year}-${month}-${day}`) : new Date();
    date2.setMonth(0)
    date2.setDate(1)
    let sub = date1 - date2
    let days = Math.ceil(sub / (24 * 60 * 60 * 1000))
    return Math.ceil(days / 7)
}

/**
 * 某月中的周六周日，如：{[1, ...], [2, ...]}
 */
String.prototype.offDaysOfMonth = function () {
    let date = new Date(this);
    let year = date.getFullYear();
    let month = date.getMonth();
    let allDay = new Date(year, month+1, 0).getDate(); // 获取当月总天数
    let saturdays = [];
    let sundays = [];
    for (let i = 1; i <= allDay; i++) {
        let week = new Date(year, month, i).getDay();
        if (week == 0) {
            sundays.push(i);
        } else if (week == 6) {
            saturdays.push(i);
        }
    }
    return { sundays, saturdays };
}

/**
 * 某月中的周六周日，如：{['2023-05-01', ...], ['2023-05-02', ...]}
 */
String.prototype.offDatesOfMonth = function () {
    let date = new Date(this);
    let year = date.getFullYear();
    let month = date.getMonth();
    let allDay = new Date(year, month+1, 0).getDate(); // 获取当月总天数
    let saturdays = [];
    let sundays = [];
    for (let i = 1; i <= allDay; i++) {
        let week = new Date(year, month, i).getDay();
        let str = `${year}-${(month+1) < 10 ? '0'+(month+1): (month+1)}-${i < 10 ? '0'+i: i}`
        if (week == 0) {
            sundays.push(str);
        } else if (week == 6) {
            saturdays.push(str);
        }
    }
    return { sundays, saturdays };
}

/*----------------------字符串截取-----------------------*/

/**
 * 获取字符串的最后num个字符
 */
String.prototype.laststr = function (num = 2) {
    if (!this) {
        return ''
    }
    if (this.length <= num) {
        return this;
    }
    return this.substr(this.length - num, this.length);
}

/**
 * 获取字符串的前面num个字符
 */
String.prototype.startstr = function (num = 2) {
    if (!this) {
        return ''
    }
    if (this.length <= num) {
        return this;
    }
    return this.substr(0, num);
}

/*----------------------字符串转换-----------------------*/

/**
 * 首字转换为16进制颜色
 */
String.prototype.firstToColor = function () {
    let firstName = this.substr(0, 1);
    let str = '';
    for (let i = 0; i < firstName.length; i++) {
        str += parseInt(firstName[i].charCodeAt(0), 10).toString(16);
    }

    // 非汉字首字，默认蓝色
    if (str.length < 4) {
        return '#0081ff';
    }
    return '#' + str.slice(1, 4);
}

/**
 * 删除小数点后面多余的0
 */
String.prototype.cutZero = function () {
    //拷贝一份 返回去掉零的新串
    let newstr = this;
    //循环变量 小数部分长度
    let leng = this.length - this.indexOf(".") - 1
    //判断是否有效数
    if (this.indexOf(".") > -1) {
        //循环小数部分
        for (let i = leng; i > 0; i--) {
            //如果newstr末尾有0
            if (newstr.lastIndexOf("0") > -1 && newstr.substr(newstr.length - 1, 1) == 0) {
                let k = newstr.lastIndexOf("0");
                //如果小数点后只有一个0 去掉小数点
                if (newstr.charAt(k - 1) == ".") {
                    return newstr.substring(0, k - 1);
                } else {
                    //否则 去掉一个0
                    newstr = newstr.substring(0, k);
                }
            } else {
                //如果末尾没有0
                return newstr;
            }
        }
    }
    return this;
}

/**
 * 转换字节数为字符串
 */
String.prototype.bytes2Text = function () {
    let s = this;
    if (!s) {
        return "0 B"
    }

    let size = s;
    if (type.isString(size)) {
        if (isNaN(Number(size))) {
            return size;
        } else {
            size = parseInt(s);
        }
    }

    let sizeText = "";
    if (size < 0.1 * 1024) { //如果小于0.1KB转化成B
        sizeText = size.toFixed(2) + " B";
    } else if (size < 0.1 * 1024 * 1024) { //如果小于0.1MB转化成KB
        sizeText = (size / 1024).toFixed(2) + " KB";
    } else if (size < 0.1 * 1024 * 1024 * 1024) { //如果小于0.1GB转化成MB
        sizeText = (size / (1024 * 1024)).toFixed(2) + " MB";
    } else { //其他转化成GB
        sizeText = (size / (1024 * 1024 * 1024)).toFixed(2) + " GB";
    }

    let len = sizeText.indexOf("\.");
    let dec = sizeText.substr(len + 1, 2);
    if (dec === "00") { //当小数点后为00时 去掉小数部分
        return sizeText.substring(0, len) + sizeText.substr(len + 3, 2);
    }
    return sizeText;
}

/*----------------------------file--------------------------*/

/**
 * 获取文件名称
 * @param {Boolean} withType 带文件类型后缀
 */
String.prototype._getFileName = function (withType = true) {
    const parts = this.split("/");
    const namePart = parts[parts.length - 1];
    if (withType) {
        return namePart;
    }

    const pointIndex = namePart.lastIndexOf(".");
    if (pointIndex !== -1) {
        return namePart.substring(0, pointIndex);
    } else {
        return namePart;
    }
}

/**
 * 获取文件名称
 */
String.prototype.getFileName = function () {
    return this._getFileName();
}

/**
 * 获取文件名称，不包含文件类型
 * @param {Object} path
 */
String.prototype.getFileNameOnly = function () {
    return this._getFileName(false);
}

/**
 * 获取文件类型，不包含点
 */
String.prototype.getFileType = function () {
    const index = this.lastIndexOf(".");
    if (index !== -1) {
        return this.substr(index + 1);
    }
    return this;
}

/*--------------------合法检测----------------------*/
/**
 * 检测邮箱合法性
 */
String.prototype.isEmail = function () {
	return RegExp(/^([a-zA-Z0-9]+[_|\_|\.]?)*[a-zA-Z0-9]+@([a-zA-Z0-9]+[_|\_|\.]?)*[a-zA-Z0-9]+\.[a-zA-Z]{2,3}$/).test(this)
}

/**
 * 检测手机合法性
 */
String.prototype.isMobile = function () {
	return RegExp(/^1[345789]\d{9}$/).test(this)
}

/**
 * 校验IP地址
 */
String.prototype.isIpAddress = function () {
    return /^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$/.test(this);
}

/**
 * 校验域名
 */
String.prototype.isDomain = function () {
    return /[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})+\.?/.test(this);
}

/**
 * 验证是否为十六进制颜色字符串
 */
String.prototype.hexColor = function () {
    return /^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/.test(this);
}

/**
 * 必须由字母和数字组成，可有特殊符号，密码长度6-18位
 */
String.prototype.checkPassword = function () {
    return /^(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z\d!@#$%^&*]{6,18}$/.test(this);
}

/**
 * 是否为纯数字
 */
String.prototype.isNumber = function () {
    return /^\d+$/.test(this);
}

/**
 * 字母和数字组合
 */
String.prototype.isLetterAndNumber = function () {
    return /[^a-zA-Z0-9]/ig.test(this);
}

/**
 * 字符串是否首字母
 */
String.prototype.isFirstLetter = function () {
    return /^[A-Za-z]{1}/.test(this);
}

/*--------------------URL相关----------------------*/

/**
 * 获取url路径，不包含参数
 */
String.prototype.getUrl = function () {
    if (this.indexOf("?") == -1) {
        return this;
    }
    return this.substring(0, this.indexOf("?")); // 获取url中"?"符前的字串
}

/**
 * 获取url参数 支持中文
 */
String.prototype.getUrlParams = function () {
    let param = {}, paramStr = this;
    if (this.indexOf("?") != -1) {
        paramStr = this.substring(this.indexOf("?") + 1); // 获取url中"?"符后的字串
    }
    let params = paramStr.split("&");
    for (let i = 0; i < params.length; i++) {
        const keyValues = params[i].split("=");
        if (!isNaN(keyValues[1])) {
            param[keyValues[0]] = parseInt(keyValues[1]);
        } else if (keyValues[1] === "true") {
            param[keyValues[0]] = true;
        } else if (keyValues[1] === "false") {
            param[keyValues[0]] = false;
        } else {
            param[keyValues[0]] = unescape(keyValues[1]);
        }
    }
    return param;
}

/**
 * 添加url参数 支持中文
 */
String.prototype.urlAddParams = function (params) {
    console.log("-----this: ",this);
    return this + params.toUrlParams();
}

/**
 * 添加url参数 支持中文
 */
String.prototype.urlAddMoreParams = function (params) {
    return this + params.toUrlMoreParams();
}

export default {}