/**
 * 项目开发中页面使用的通用方法
 */
export default {
    computed: {
        /**
         * 颜色
         */
        mc_colors() {
            return this.$kit.color;
        }
    },
    methods: {
        /**
         * 显示加载UI
         */
        mixin_showLoading(title = this.$t("common.loading"), mask = true) {
            this.$kit.showLoading(title, mask)
        },
        
        /**
         * 显示消息提示框
         */
        mixin_showToast(title, duration = 2500, icon = "none") {
            this.$kit.showToast(title, duration, icon)
        },
        
        /**
         * 模态框
         */
        mixin_showModal(content, confirm, showCancel = true, title = this.$t("common.warm-prompt")) {
            this.$kit.showModal(content, confirm, showCancel, title)
        },
        
        /**
         * 裁剪字符串
         */
        mixin_last(value, len = 2) {
            if (!value) {
                return ''
            }
            
            if (value.length <= num) {
                return value;
            }
            
            return value.substr(value.length - num, value.length);
        },
        
        /**
         * 实现显示指定长度字符串，多余字符省略功能，支持头部、中间和尾部省略符号
         * 省略开始
         * @param {Object} value
         */
        mixin_startEllipsis(value, showLen = 5) {
            if (!value) {
                return ''
            }
        
            const len = value.length
            if (len <= showLen) {
                return value
            } else {
                return '...' + value.slice(len - showLen, len)
            }
        },
        
        /**
         * 省略中间
         * @param {Object} value
         */
        mixin_midEllipsis(value, startLen = 2, endLen = 3) {
            if (!value) {
                return ''
            }
        
            const len = value.length
            if (len <= startLen + endLen) {
                return value
            } else {
                return value.slice(0, startLen + 1) + '...' + value.slice(len - endLen, len)
            }
        },
        
        /**
         * 省略尾部
         * @param {String} value
         * @param {Number} showLen
         */
        mixin_endEllipsis(value, showLen = 5) {
            if (!value) {
                return ''
            }
        
            const len = value.length
            if (len <= showLen) {
                return value
            } else {
                return value.slice(0, showLen) + '...'
            }
        },
        
        /**
         * 判断value是否为空，包括空对象和空字符串
         * @param {Object} value
         */
        mixin_empty(value) {
            return this.$kit.isEmpty(value);
        },
        
        /**
         * 判断value是否为非空空
         * @param {Object} value
         */
        mixin_nonEmpty(value) {
            return !this.$kit.isEmpty(value);
        },
        
        /**
         * 判断value是否为null或undefined
         * @param {Object} value
         */
        mixin_strictEmpty(value) {
            return this.$kit.isStrictEmpty(value);
        },
        
        /**
         * 判断value是否为非null或非undefined
         * @param {Object} value
         */
        mixin_nonStrictEmpty(value) {
            return !this.$kit.isStrictEmpty(value);
        },
        
        /**
         * 删除浮点数小数点后的0
         * @param {Object} value
         */
        mixin_cutZero(value) {
            return this.$kit.cutZero(value);
        },
        
        /**
         * 名字第一个字符转换为16进制颜色
         * @param {Object} name
         */
        mixin_tranColor(name) {
            return transformer.tranColor(name);
        },
        
        /**
         * 没有更多数据
         * @param {Object} data
         */
        mixin_noMoreData(data) {
            if (this.$kit.isEmpty(data)) {
                return true;
            }
            return data.length < this.$config.pageSize;
        },
        
        /**
         * UI元素是否在可视区域
         * @param selector: 选择器
         * @param callback: 两个参数是否找到元素和是否可见（found, isVisible）
         */
        mixin_isVisible(selector, callback) {
            const {
                windowWidth,
                windowHeight
            } = this.vuex_systemInfo;
        
            const _isVisible = function(rect) {
                if (!rect) {
                    return false
                }
        
                const {
                    top,
                    bottom,
                    left,
                    right
                } = rect;
                const yInView = top < windowHeight && bottom <= windowHeight // bottom判断可确保元素垂直完全可见
                const xInView = left < windowWidth && right <= windowWidth // right判断可确保元素水平完全可见
                return yInView && xInView
            }
        
            if (callback) {
                this.$kit.getBoundingClientRect(selector).then(res => {
                    callback(res, _isVisible(res))
                })
            } else {
                return this.$kit.getBoundingClientRect(selector).then(res => {
                    return {
                        found: res,
                        isVisible: _isVisible(res)
                    }
                })
            }
        },
        
        /**
         * UI元素高度
         * @param selector: 选择器
         * @param callback: 回调
         */
        mixin_componentHeight(selector, callback) {
            if (callback) {
                this.$kit.getBoundingClientRect(selector).then(rect => {
                    callback(rect.bottom - rect.top);
                })
            } else {
                return this.$kit.getBoundingClientRect(selector).then(rect => {
                    return rect.bottom - rect.top;
                })
            }
        },
        
        /**
         * 页面滚动到指定元素
         * @param selector: 选择器
         */
        mixin_pageScrollTo(selector, duration = 200, extra = 0) {
            this.mixin_isVisible(selector, (found, isVisible) => {
                if (found) {
                    if (!isVisible) {
                        this.$kit.getRect(selector).then(res => {
                            uni.pageScrollTo({
                                scrollTop: res.top + extra,
                                duration
                            })
                        })
                    }
                }
            })
        }
    }
}
