import pageJson from "@/pages.json"

/**
 * 页面跳转方法定义
 */
export default {
    /**
     * 页面数据
     */
    data() {
        return {
            mixin_homePage: "/" + pageJson.tabBar.list[0].pagePath // 主页面
        }
    },
    
    /**
     * 页面方法
     */
    methods: {
        /**
         * 页面跳转
         * @param {Object} jumpPath
         * @param {Object} options
         */
        mixin_navigateTo(jumpPath, options) {
            this.$kit.navigateTo(jumpPath, options)
        },
        
        /**
         * 页面跳转到tab页面
         * @param {Object} jumpPath
         * @param {Object} options
         */
        mixin_switchTab(jumpPath, options) {
            this.$kit.switchTab(jumpPath, options)
        },
        
        /**
         * 页面重定向
         * @param {Object} jumpPath
         * @param {Object} options
         */
        mixin_redirectTo(jumpPath, options) {
            this.$kit.redirectTo(jumpPath, options)
        },
        
        /**
         * 关闭所有页面，打开到应用内的某个页面
         * @param {Object} jumpPath
         * @param {Object} options
         */
        mixin_reLaunch(jumpPath, options) {
            uni.reLaunch({ url: jumpPath + (options ? options.toUrlParams: "") });
        },
        
        /**
         * 页面跳转
         */
        mixin_toPage(jumpPath, options, link = "navigateTo") {
            switch (link) {
                case "navigateTo":
                    this.mixin_navigateTo(jumpPath, options);
                    break;
                case "switchTab":
                    this.mixin_switchTab(jumpPath, options);
                    break;
                case "redirectTo":
                    this.mixin_redirectTo(jumpPath, options);
                    break;
                default:
                    this.mixin_navigateTo(jumpPath, options);
                    break;
            }
        },
        
        /**
         * 回调父页面方法并返回上一页
         */
        mixin_back(...params) {
            this.$kit.callParentPage.apply(null, params);
            uni.navigateBack();
        },
        
        /**
         * 已添加回调并返回上一页，不显示toast
         */
        mixin_backAdded(...params) {
            this.mixin_back('mixin_added', ...params)
        },
        
        /**
         * 已更新回调并返回上一页，不显示toast
         */
        mixin_backUpdated(...params) {
            this.mixin_back('mixin_updated', ...params)
        },
        
        /**
         * 已删除回调并返回上一页，不显示toast
         */
        mixin_backDeleted(...params) {
            this.mixin_back('mixin_deleted', ...params)
        },
        
        /**
         * 需要登录的页面
         */
        mixin_needLogin(url = this.$kit.getPage().route) {
            const {
                uniIdRouter: {
                    needLogin: needLoginPages
                }
            } = pageJson;
            let needLogin = false;
            for (let i = 0; i < needLoginPages.length; i++) {
                const needLoginPage = needLoginPages[i];
                if (RegExp(needLoginPage).test(url)) {
                    needLogin = true;
                    break;
                }
            }
            return needLogin;
        },
        
        /**
         * 获取登录页面路径
         */
        mixin_getLoginUrl() {
            const {
                uniIdRouter: {
                    loginPage: loginUrl
                }
            } = pageJson;
            return loginUrl.startsWith("/") ? loginUrl: `/${loginUrl}`;
        },
        
        /**
         * 跳转登录页面
         */
        mixin_toLogin(options) {
            this.mixin_toPage(this.mixin_getLoginUrl(), options);
        },
        
        /**
         * 显示登录对话框
         * @param {Object} url 检测跳转路径是否要求登录
         */
        mixin_showLogin(url) {
            // 如果该页面未定义需要登录，直接返回false
            if (url && !this.mixin_needLogin(url)) {
                return false;
            }
            
            if (!this.vuex_user_isTokenValid) { // store下vuex状态管理，modules/user中定义
                setTimeout(_ => {
                    this.$kit.showModal2(this.$t('common.to-login'), 
                        this.$t('button.cancel'), 
                        this.$t('login.login'), 
                        _ => {
                        this.mixin_toLogin({
                            uniIdRedirectUrl: encodeURIComponent(this.$kit.getPage().route)
                        })
                    });
                }, 300);
            }
            return !this.vuex_user_isTokenValid;
        }
    }
}