/**
 * 子组件调用父页面方法
 */
import mixinJump from "../common/jump.js";

export default {
    mixins: [mixinJump],
    props: {
        // 跳转方式
        link: {
            type: [Boolean, String],
            default: false
        },
        // 跳转页面路径
        to: {
            type: String,
            default: ""
        },
        // 是否开启点击反馈
        clickable: {
            type: Boolean,
            default: true
        },
        // 是否禁用
        disabled: {
            type: Boolean,
            default: false
        }
    },
    methods: {
        /**
         * 长按事件
         */
        mixin_onLongpress() {
            if (!this.disabled) {
                this.$emit('longpress');
            }
        },
        
        /**
         * 点击事件
         */
        mixin_onClick() {
            // 如果需要跳转，popup弹出框消失后再跳转
            if (this.to) {
                if (this.$config.uniIntercept || !this.mixin_showLogin(this.to)) {
                    if (this.clickable || this.link) {
                        setTimeout(_ => this.mixin_jumpToPage(), 200);
                    } else {
                        this.mixin_jumpToPage();
                    }
                }
            } else if (!this.disabled) {
                if (this.clickable || this.link) {
                    setTimeout(_ => this.$emit('click'), 200);
                } else {
                    this.$emit('click');
                }
            }
        },
        
        /**
         * 页面跳转
         */
        mixin_jumpToPage() {
            const linkTypes = ['navigateTo', 'redirectTo', 'reLaunch', 'switchTab'];
            const linkType = linkTypes.includes(this.link) ? this.link: "navigateTo";
            this.mixin_toPage(this.to, null, linkType);
        }
    }
}