/**
 * 组件基础
 */
import mixinCommon from "../common/common.js";

export default {
    mixins: [mixinCommon],
    props: {
        // 自定义样式类，注意：传递数组时，每个元素只能是字符串
        customClass: {
            type: [String, Array],
            default: ''
        },
        // 自定义样式
        customStyle: {
            type: Object,
            default: function() {
                return {}
            }
        }
    },
    computed: {
        /**
         * 处理自定义类属性
         * 注意：传递数组时，每个元素只能是字符串
         */
        mc_customClass() {
            if (this.$kit.isArray(this.customClass)) {
                return this.customClass.join(" ");
            }
            return this.customClass;
        },
        
        /**
         * 样式对象
         */
        mc_customStyle() {
            return (key, value) => {
                if (key && value) {
                    return {
                        key: value,
                        ...this.customStyle
                    }
                } else {
                    return this.customStyle;
                }
            }
        }
    },
    methods: {
        /**
         * 获取父元素实例
         */
        mixin_getParent(name) {
            let parent = this.$parent;
            let parentName = parent.$options.name;
            while (parentName !== name) {
            	parent = parent.$parent;
            	if (!parent) return false
            	parentName = parent.$options.name;
            }
            return parent;
        }
    }
}