/**
 * 子组件调用父页面方法
 */
import mixinJump from "../common/jump.js";

export default {
    mixins: [mixinJump],
    props: {
        // 列表数据
        list: {
            type: Array,
            default() {
                return []
            }
        }
    },
    methods: {
        /**
         * 长按事件
         */
        mixin_onLongpress(index) {
            const item = this.list[index];
            if (!this.disabled) {
                this.mixin_callPage("lpHandler", index, index);
                this.$emit('longpress');
            }
        },
        
        /**
         * 响应点击事件
         * @param {Object} index
         */
        mixin_onClick(index) {
            // 如果需要跳转，popup弹出框消失后再跳转
            const item = this.list[index];
            if (item.to) {
                console.log("item.to: ",item.to);
                if (this.$config.uniIntercept || !this.mixin_showLogin(item.to)) {
                    if (item.clickable) {
                        setTimeout(_ => this.mixin_list_jumpToPage(index), 350);
                    } else {
                        this.mixin_list_jumpToPage(index); 
                    }
                }
            } else {
                this.mixin_callPage("handler", index, index);
                this.$emit('click', index);
            }
        },
        
        /**
         * 调用页面回调处理方法
         * @param {Object} method
         * @param {Object} index
         * @param {Object} value
         */
        mixin_callPage(method, index, value) {
            // 通过全局数据保存的当前页面调用回调方法，也可以使用this.$u.$parent()获取组件的顶层组件即页面对象
            const page = getApp().globalData.curPage;
            const handler = this.list[index][method];
            if (handler) {
                if (this.$kit.isFunction(handler)) { // 传递的方法
                    // 当采用回调方式处理点击事件时，只需要传递事件产生的数据，不需要索引参数
                    handler(value);
                } else if (this.$kit.isFunction(page[handler])) { // 传递的方法名称
                    // 当采用回调方式处理点击事件时，只需要传递事件产生的数据，不需要索引参数
                    page[handler](value);
                }
            }
        },
        
        /**
         * 跳转页面
         * @param {Object} index
         */
        mixin_list_jumpToPage(index) {
            this.mixin_jumpToPage(this.list[index].to, this.list[index].linkType || this.list[index].link || 'navigateTo');
        },
        
        /**
         * 内部跳转方法
         * @param {Object} to
         * @param {Object} type
         */
        mixin_jumpToPage(to, type = "navigateTo") {
            const linkType = type;
            const linkTypes = ['navigateTo', 'redirectTo', 'reLaunch', 'switchTab'];
            if (linkTypes.includes(linkType)) {
                this.mixin_toPage(to, null, linkType);
            }
        }
    }
}