import auth from './auth.js'
import page from './page.js'
import request from "./request/index.js"
import scroll from "./scroll/index.js"
import pageList from "./list/index.js"
import component from "./component/index.js"

/**
 * 借鉴uView挂载$u的方式，可在vue页面使用uni.$mixer混入
 * 注意：这里的uni.$mixer只能在vue页面使用，因为在main.js中安装此插件时是全局混入，这对nvue
 * 是无效的，如果在nvue页面直接使用uni.$mixer.nvue，实际上只是混入了nvue的相关数据和方法，
 * 而没有重新挂载各个组件，导致页面初始化使用相关组件时失败，解决办法：
 * 1. 在nvue页面手动再次导入“@/common/mixin/nvue.js”文件
 * 2. 导入已导入nvue.js文件的文件
 */
uni.$mixer = {
    auth,
    // page, // 安装时已全局混入所有页面
    ...request,
    ...scroll,
    ...pageList,
    ...component
};

/**
 * 通过插件方式使用，当然也可以直接在main.js中给Vue原型赋值
 */
// #ifdef VUE2
const install = Vue => {
    // 默认混入page, 页面基本混入
    Vue.mixin(page);
}
// #endif
// #ifdef VUE3
const install = app => {
    // 默认混入parent, 页面基本混入
    app.mixin(page);
}
// #endif

/**
 * 安装插件后，请使用this.$mixer混入
 */
export default {
    install
}