/**
 * 标准的标签页列表页面的基本组件
 */

import mixinBase from '../request/base.js'

export default {
    /**
     * 混入列表更新、下拉刷新上拉加载功能
     */
    mixins: [mixinBase],

    /**
     * 数据
     */
    data() {
        return {
            // 当前tab索引
            mv_tabIndex: 0,
            // swiper索引
            mv_swiperCurrent: 0, // uview tabSwiper
            // tab数据
            mv_tabs: [],
            // 是否根据tabbar索引变化自动更新页面标题
            mv_autoTitle: false
        }
    },

    computed: {
        /**
         * 当前标签
         */
        curTab() {
            return this.mv_tabs[this.mv_tabIndex]
        }
    },

    watch: {
        /**
         * 监控mv_tabIndex属性的变化，获取检索后的业务数据
         * @param {Object} newValue
         * @param {Object} oldValue
         */
        mv_tabIndex(newValue, oldValue) {
            // 自动更新页面标题为tab项的name
            if (this.mv_autoTitle) {
                this.navTitle = this.curTab.name;
            }
            // 切换tab，返回false表示继续执行，返回true表示处理完成不在继续执行
            if (this.mixin_onSwitch()) {
                return;
            } 
            // 加载数据
            if (!this.curTab.loaded) {
                this.mixin_reloadData();
            }
        }
    },

    methods: {
        /**
         * 切换tab，返回false表示继续执行，返回true表示处理完成不在继续执行
         */
        mixin_onSwitch() {
            return false;
        },
        
        /**
         * 添加页面的添加动作完成后回调
         */
        mixin_doAdded(tabIndex) {
            if (!this.$kit.isStrictEmpty(tabIndex)) {
                this.mv_tabIndex = tabIndex
                // uview的tabSwiper控件使用
                this.mv_swiperCurrent = tabIndex
            }
        },
        
        /**
         * 添加页面的更新动作完成后回调
         */
        mixin_doUpdated(target) {
            for (let i = 0; i < this.curTab.data.length; i++) {
                const item = this.curTab.data[i];
                if (item.id == target.id) {
                    const newItem = { ...item, ...target }
                    this.curTab.data.splice(i, 1, this.mixin_handleItem ? this.mixin_handleItem(newItem): newItem);
                    break;
                }
            }
        },
        
        /**
         * 详情页面的删除动作完成后回调
         */
        mixin_doDeleted(itemId) {
            for (let i = 0; i < this.curTab.data.length; i++) {
                if (this.curTab.data[i].id == itemId) {
                    this.curTab.data.splice(i, 1);
                    break;
                }
            }
        },
        
        /**
         * 判断是否还有更多业务数据
         */
        mixin_hasMoreData() {
            return this.curTab.status == 'loading'
        },
        
        /**
         * 下拉刷新时重置页面数据
         */
        mixin_resetForRefresh() {
            this.mixin_resetCurTab();
        },
        
        /**
         * 更新数据时重置数据
         */
        mixin_resetForUpdate() {
            this.mixin_resetCurTab();
        },
        
        /*---------------和swiper配合使用，如uview的tabSwiper，开始----------------*/
        
        /**
         * uview的tabSwiper使用
         * @param {Number} current
         */
        mixin_tabChanged(current) {
            this.mv_swiperCurrent = current;
            // #ifdef APP-NVUE
            this.mv_tabIndex = current;
            // #endif
        },
        
        /**
         * swiper-item左右移动，通知mv_tabs的滑块跟随移动
         * @param {Object} e
         */
        mixin_transition(e) {
            let dx = e.detail.dx;
            if (this.$refs.uTabs) {
                this.$refs.uTabs.setDx(dx);
            }
        },
        
        /**
         * 由于swiper的内部机制问题，快速切换swiper不会触发dx的连续变化，需要在结束时重置状态,
         * swiper滑动结束，分别设置tabs和swiper的状态
         * @param {Object} e
         */
        mixin_animationFinish(e) {
            let current = e.detail.current;
            if (this.$refs.uTabs) {
                this.$refs.uTabs.setFinishCurrent(current);
            }
            this.mv_tabIndex = current;
            this.mv_swiperCurrent = current;
        },
        
        /*---------------和swiper配合使用，如uview的tabSwiper，结束----------------*/
        
        /**
         * tab一般会和swiper一起使用
         * @param {Object} e
         */
        mixin_onSwiperChanged(e) {
            this.mv_tabIndex = e.detail.current
            // #ifdef APP-NVUE
            this.mv_swiperCurrent = e.detail.current
            // #endif
        },
        
        /**
         * scrollview滑动到底部加载更多数据
         */
        mixin_onScrollToLower() {
            if (!this.mv_loading && this.mixin_hasMoreData()) {
                this.mixin_loadData(false)
            }
        },

        /**
         * 重置列表参数
         */
        mixin_resetPage() {
            this.mv_tabs.forEach(tab => {
                tab.current = 0
                tab.status = 'loading'
                tab.loaded = false
            })
        },
        
        /**
         * 重置列表参数
         */
        mixin_resetOtherTabs() {
            this.mv_tabs.forEach((tab, index) => {
                if (index != this.mv_tabIndex) {
                    tab.current = 0
                    tab.status = 'loading'
                    tab.loaded = false
                }
            })
        },

        /**
         * 重置当前页签列表参数
         */
        mixin_resetCurTab() {
            this.curTab.current = 0
            this.curTab.status = 'loading'
            this.curTab.loaded = false
        },

        /**
         * 重新加载业务数据
         */
        mixin_reloadData(showLoading = true) {
            this.mixin_resetCurTab()
            this.mixin_loadData(showLoading)
        },
        
        /**
         * 请求结束，可能是异常，该方法主要用于设置相关标志和隐藏加载视图
         */
        mixin_reqEnd(showLoading = true, exception = false) {
            this.curTab.loaded = true
            
            // 如果是异常
            if (exception) {
                // #ifdef APP-NVUE
                this.mixin_tablist_reqEnd(false)
                // #endif
                // #ifndef APP-NVUE
                this.curTab.status = 'erro'
                // #endif
            }
            
            // 下拉加载不会显示modal加载框，但上拉加载更多也不会显示，此时不需要uni.stopPullDownRefresh，不多调用也不影响页面逻辑
            if (showLoading) {
                this.mv_loading = false
                uni.hideLoading()
                // #ifdef APP-NVUE
                this.mixin_tablist_reqEnd(true)
                // #endif
            } else {
                // #ifdef APP-NVUE
                this.mixin_tablist_reqEnd(true)
                // #endif
                // #ifndef APP-NVUE
                uni.stopPullDownRefresh()
                // #endif
            }
        },
        
        /**
         * 格式化数据项
         * @param {Object} item
         */
        mixin_handleItem(item) {
            return item;
        },
        
        /**
         * 格式化数据
         * @param {Object} res
         */
        mixin_handleData(res) {
            return res.data.data.map(item => this.mixin_handleItem(item))
        },

        /**
         * 业务数据请求返回正常处理
         */
        mixin_reqThen(res, showLoading = true) {
            if (this.$kit.checkStatus(res, false)) {
                if (this.curTab.mv_current == 0) {
                    this.curTab.mv_data = this.mixin_handleData(res);
                } else {
                    this.curTab.mv_data = [ ...this.curTab.mv_data, ...this.mixin_handleData(res) ];
                }
                
                if (this.$kit.isArray(res)) {
                    if (!res.map(item => this.mixin_noMoreData(item.data.data)).includes(false)) {
                        this.curTab.mv_status = "nomore";
                    }
                } else if (this.mixin_noMoreData(res.data.data)) {
                    this.curTab.mv_status = "nomore";
                }
                this.curTab.mv_current += 1
            } else {
                this.curTab.mv_status = "erro";
            }
            
            this.mixin_reqEnd(showLoading)
        },
        
        /**
         * 加载业务数据
         */
        mixin_loadData(showLoading = true) {
            /**
             * 其他准备工作，如登录检查、权限控制等
             */
            if (!this.mixin_loadBefore()) {
                return
            }
        
            // 如果有正在执行的任务，不会重复执行
            if (this.mv_loading) {
                return
            }
        
            if (!this.mixin_request) {
                return
            }
            
            if (showLoading) {
                this.mv_loading = true
                this.$kit.showLoading()
            }
        
            this.mixin_httpCall(this.mixin_request).then(res => {
                this.mixin_reqThen(res, showLoading)
            }).catch(res => {
                this.mixin_reqCatch(res, showLoading)
            })
        }
    }
}
