/**
 * nvue页面相关混入
 */
// #ifdef APP-NVUE
import Vue from 'vue'

// 引入全局uView，yarn安装
// import uView from 'uview-ui'
// Vue.use(uView)

// http接口API抽离，免于写url或者一些固定的参数
// Vue.prototype.$http
import http from '@/common/http/http.install.js'
Vue.use(http)

// storage的存取代理插件
// Vue.prototype.$storage
import storageProxy from "@/libs/storage/storage.js"
Vue.use(storageProxy)

/** -------------------------直接混入-------------------------------
 * 说明：混入 (mixin) 提供了一种非常灵活的方式，来分发 Vue 组件中的可复用功能。一个混入对象可以包含任意组件选项。
 * 当组件使用混入对象时，所有混入对象的选项将被“混合”进入该组件本身的选项。
 * 注意：当组件和混入对象含有同名选项时，这些选项将以恰当的方式进行“合并”
 * （1）数据对象在内部会进行递归合并，并在发生冲突时以组件数据优先
 * （2）同名钩子函数将合并为一个数组，因此都将被调用。另外，混入对象的钩子将在组件自身钩子之前调用
 *     其中包括：周期钩子、watch方法等
 * （3）methods、components 和 directives，将被合并为同一个对象。两个对象键名冲突时，取组件对象的键值对
 */

// 引入uView提供的对vuex的简写法文件，vuex插件也在其中安装
let vuexMixin = require('@/store/store.mixin.js')
Vue.mixin(vuexMixin)

/** ---------------------------prototype挂载-----------------------------
 * 全局数据的一种实现方式
 */

// 混入过滤器, 让每个页面都具有该过滤器组件选项
import mixinFilter from "./filter.js"
Vue.mixin(mixinFilter);

// 挂载工具套件
import kit from '@/common/kit/index.js'
Vue.prototype.$kit = kit
// 主要用于uView相关组件的颜色配置，因使用频率较高这里直接挂载到Vue原型
Vue.prototype.$color = kit.color

// 全局配置
import config from '@/config.js'
Vue.prototype.$config = config

// 全端兼容剪贴板
import {setClipboardData, getClipboardData} from 'uni-clipboard'
Vue.prototype.$clipboard = {
    setClipboardData,
    getClipboardData
}

/** ----------------------------使用vuex--------------------------------*/
// vuex对store的封装
import store from '@/store'
Vue.prototype.$vstate = store.state

// #endif

export default {
    // #ifdef APP-NVUE
    
    data() {
        return {
            mv_windowHeight: 0
        }
    },
    
    methods: {
        /**
         * 初始化窗口高度
         */
        mixin_initWindowHeight() {
            this.mv_windowHeight = this.$u.sys().windowHeight;
            // 可在nvue页面的子组件中使用，每个页面都会动态初始化一次
            getApp().globalData.windowHeight = this.mv_windowHeight;
        }
    },
    
    onLoad() {
        /**
         * main.js中执行了store的混入，this.$u已经被使用，这里安装挂载会失败，
         * 由于uView在main.js中安装时已经将$u挂载到uni上，因此可以通过onLoad中
         * 通过uni.$u给this.$u初始化
         */
        this.$u = uni.$u;
        
        // 初始化窗口高度，主要用于nvue的全屏控制
        this.mixin_initWindowHeight();
    }
    // #endif
}