/**
 * 实现详情页面网络请求，可随页面刷新操作而重新请求网络数据
 */
// vue页面parent默认混入，因此这里不需要再导入，nvue页面无法默认混入
import mixinInit from './init.js'
// #ifndef MP-WEIXIN
// import mixinWatermark from '../watermark.js';
// #endif
// import mixinLogin from '../login.js';

export default {
    mixins: [
        mixinInit,
        // #ifndef MP-WEIXIN
        // mixinWatermark, 
        // #endif
        // mixinLogin
    ],

    /**
     * 基础数据
     */
    data() {
        return {
            // 页面数据加载中, mv（mixin variable）
            mv_loading: false,
            // 页面数据已加载
            mv_loaded: false,
            // 页面数据
            mv_data: [],
            // 更新当前页面
            mv_updatePage: false,
            // 级联刷新上级页面
            mv_updateParentPage: false,
            // 下拉动作正在刷新
            mv_refreshing: false
        }
    },

    methods: {
        /**
         * 添加页面的添加动作完成后回调
         * @param {Object} index tab-list/tabbar-list页面使用
         */
        mixin_added(index) {
            // 强制更新当前页面
            this.mv_updatePage = true
            
            // 添加更新时，额外操作
            if (this.mixin_doAdded) {
                this.mixin_doAdded(index);
            }
        },
        
        /**
         * 添加页面的更新动作完成后回调
         */
        mixin_updated(updateItem) {
            if (updateItem) { // 直接更新data中的数据
                // 添加更新时，额外操作
                if (this.mixin_doUpdated) {
                    this.mixin_doUpdated(updateItem);
                }
            } else {
                // 强制更新当前页面
                this.mv_updatePage = true
            }
        },
        
        /**
         * 详情页面的删除动作完成后回调
         */
        mixin_deleted(delId) {
            if (delId) {
                // 添加更新时，额外操作
                if (this.mixin_doDeleted) {
                    this.mixin_doDeleted(delId);
                }
            } else {
                this.mv_updatePage = true;
            }
        },
        
        /**
         * 重置列表参数，不调用不会报错
         */
        mixin_resetPage() {
            this.mv_loaded = false
        },
        
        /**
         * 更新数据时重置数据，在list/tab-list/tabbar-list中实现
         */
        mixin_resetForUpdate() {
            
        },
        
        /*------------------下拉刷新和上拉触底加载更多--------------*/
        /**
         * 处理刷新操作
         * 返回：false (可继续执行)，
         * 返回：true (不需要继续执行)
         */
        mixin_handleRefresh() {
            return false
        },
        
        /**
         * 加载更多操作
         * 返回：false (可继续执行)，
         * 返回：true (不需要继续执行)
         */
        mixin_handleReachBottom() {
            return false
        },
        
        /**
         * 下拉刷新时重置页面数据，在list/tab-list/tabbar-list中实现
         */
        mixin_resetForRefresh() {
            
        },
        
        /*------------------加载数据--------------*/
        /**
         * 重新加载业务数据
         */
        mixin_reloadData(showLoading = true) {
            this.mixin_resetPage()
            this.mixin_loadData(showLoading)
        },

        /**
         * 请求结束，可能是异常，该方法主要用于设置相关标志和隐藏加载视图
         */
        mixin_reqEnd(showLoading = true, exception = false) {
            this.mv_loaded = true
            this.mv_loading = false
            
            if (showLoading) {
                uni.hideLoading()
            } else {
                uni.stopPullDownRefresh()
            }
        },

        /**
         * 处理返回的列表业务数据
         * @param {Object} res
         */
        mixin_handleData(res) {
            return res.data.data;
        },

        /**
         * 业务数据请求返回正常处理
         */
        mixin_reqThen(res, showLoading = true) {
            if (this.$kit.checkStatus(res, false)) {
                this.mv_data = this.mixin_handleData(res);
            }
            this.mixin_reqEnd(showLoading);
        },

        /**
         * 业务数据请求返回异常处理
         */
        mixin_reqCatch(res, showLoading = true) {
            this.$kit.checkStatus(res, false);
            this.mixin_reqEnd(showLoading, true);
        },

        /**
         * 其他准备工作，如登录检查、权限控制等
         * 返回true表示继续执行，false表示不再执行后续操作
         */
        mixin_loadBefore() {
            return !this.mv_initialized;
        },

        /**
         * 加载业务数据, 不建议重写（规则）
         */
        mixin_loadData(showLoading = true) {
            /**
             * 其他准备工作，如登录检查、权限控制等
             */
            if (this.mixin_loadBefore()) {
                // 和refresh混入搭配使用
                if (this.mv_refreshing) { 
                    this.mv_refreshing = false;
                    uni.stopPullDownRefresh()
                }
                return
            }

            // 如果有正在执行的任务，不会重复执行
            if (showLoading && this.mv_loading) {
                return
            }
            
            // 是否存在待处理的网络请求
            if (!this.mixin_request) {
                if (this.mv_refreshing) {
                    this.mv_refreshing = false;
                    uni.stopPullDownRefresh()
                }
                return
            }
            
            if (showLoading) {
                uni.showLoading()
            }
            
            // 防止重复请求
            this.mv_loading = true
            
            // 网络请求
            this.mixin_httpCall(this.mixin_request).then(res => {
                this.mixin_reqThen(res, showLoading)
            }).catch(res => {
                this.mixin_reqCatch(res, showLoading)
            })
        }
    },

    /**
     * 加载页面钩子函数，可与页面内的onLoad一起执行，此处的钩子会先执行
     */
    onLoad(option) {
        // 加载业务数据，如果未提供requests则直接返回
        // #ifdef MP-WEIXIN
        if (this.$kit.isAndroid()) {
            setTimeout(_ => this.mixin_loadData(), 100);
        } else {
            this.mixin_loadData()
        }
        // #endif
        // #ifndef MP-WEIXIN
        this.mixin_loadData()
        // #endif
    },
    
    /**
     * 如果页面被标记为需要更新标志，则重新加载数据
     */
    onShow() {
        if (this.mv_updatePage) {
            this.mv_updatePage = false;
            
            // 重置当前页面
            this.mixin_resetForUpdate();
            
            // 更新操作
            if (this.mixin_doUpdate) {
                this.mixin_doUpdate();
            } else {
                this.mixin_loadData(false);
            }
            
            // 重启页内动画标志
            if (this.mixin_enableAnimation) {
                this.mixin_enableAnimation();
            }
            
            // 刷新上级页面
            if (this.mv_updateParentPage) {
                this.mixin_updateParentPage();
            }
        }
    },
    
    /**
     * 下拉刷新
     */
    onPullDownRefresh() {
        // 返回：true (可继续执行)，false (不需要继续执行)
        if (!this.mixin_handleRefresh()) {
            this.mv_refreshing = true;
            this.mixin_resetForRefresh()
            this.mixin_loadData(false)
        } else {
            uni.stopPullDownRefresh()
        }
    },
    
    /**
     * 上拉加载
     */
    onReachBottom() {
        // 返回：true (未处理完成，需要继续执行)，false (处理完成，不需要继续执行)
        if (!this.mixin_handleReachBottom() 
            && this.mixin_hasMoreData()) {
            this.mixin_loadData(false)
        }
    }
}
