/**
 * 横竖屏
 */
import config from "@/config.js"

/**
 * 适配华为Welink轻应用
 */
export default {
    data() {
        return {
            mv_autoRotate: false, // 横屏时设置为自动翻转，从子页面返回该页面自动变为横屏
            mv_vertical: true, // 横竖屏标志
            mv_rotating: false, // 控制翻转屏幕时刷新页面
            mv_scrollTop: 0 // 记录页面滑动位置
        }
    },
    
    watch: {
        mv_rotating(newValue, oldValue) {
            if (!this.mv_rotating) {
                setTimeout(_ => {
                    uni.pageScrollTo({
                        scrollTop: this.mv_scrollTop,
                        duration: 0
                    })
                }, 200)
            }
        }
    },
    
    computed: {
        /**
         * 指定尺寸的正方形
         * @param {Object} num
         */
        mixin_square() {
            return (num) => {
                return `square-${this.mv_vertical ? num: num/2}vw`;
            }
        }
    },

    methods: {
        /**
         * 强制更新，可使用mv_rotating标志
         */
        mixin_forceUpdate() {
            this.mv_rotating = true
            setTimeout(_ => {
                this.mv_rotating = false
            }, 200)
        },

        /**
         * 翻转屏幕
         */
        mixin_onRotateScreen() {
            if (this.mv_vertical) {
                this.mv_autoRotate = true
                this.mixin_horizontalScreen();
            } else {
                this.mv_autoRotate = false
                this.mixin_verticalScreen();
            }
        },

        /**
         * 更新方向标志
         * @param {Object} vertical
         */
        mixin_updateVertical(vertical) {
            this.mv_vertical = vertical;
            getApp().globalData.vertical = vertical;
            this.mixin_forceUpdate()
        },

        /**
         * 竖屏
         */
        mixin_verticalScreen() {
            // #ifdef H5
            if (config.platform == "welink" && !this.mv_vertical) {
                HWH5.offHorizontalScreen();
                this.mixin_updateVertical(true);
            }
            // #endif
        },

        /**
         * 横屏
         */
        mixin_horizontalScreen() {
            // #ifdef H5
            if (config.platform == "welink" && this.mv_vertical) {
                HWH5.horizontalScreen();
                this.mixin_updateVertical(false);
            }
            // #endif
        }
    },

    /**
     * 加载
     */
    onLoad() {
        // 横屏
        if (this.mv_autoRotate) {
            this.mixin_horizontalScreen()
        }
    },

    /**
     * 隐藏
     */
    onShow() {
        // 竖屏
        if (this.mv_autoRotate) {
            this.mixin_horizontalScreen();
        }
    },
    
    /**
     * 记录页面滚动
     * @param {Object} e
     */
    onPageScroll(e) {
        if (!this.mv_rotating) {
            this.mv_scrollTop = e.scrollTop
        }
    },
    
    /**
     * 隐藏
     */
    onHide() {
        // 竖屏
        if (this.mv_autoRotate) {
            this.mixin_verticalScreen();
        }
    },

    /**
     * 卸载
     */
    onUnload() {
        // 竖屏
        if (this.mv_autoRotate) {
            this.mixin_verticalScreen();
        }
    }
}
