import mock from "@/common/request/utils/mock.js"

/**
 * 发送验证码
 */
const sendVerificationCode = async (mobile) => {
	return await uni.$http.GET(`/sys/app/message/common/verifyCode/send/${mobile}/ext/api`);
}

/**
 * 验证验证码
 */
const verifyCodeValid = async (mobile, verificationCode) => {
	return await uni.$http.GET(`/sys/app/message/common/verifyCode/check/${mobile}/${verificationCode}/ext/api`);
}

/**
 * 手机号是否已注册
 */
const hasMobileRegistered = async (mobile) => {
	return await uni.$http.GET(`/sys/app/phone/${mobile}/validate/ext/api`);
}

/**
 * 表格
 */
const loadTableData = async ({ date = "2023-05"}) => {
    const dates = date.datesOfMonth();
    return mock.mockNetwork([1, 2, 3, 4, 5, 6, 7, 8, 9, 10].map(item => {
        const projectList = []
        for (var i = 0; i < dates.length; i++) {
            if (i%2 == 0) {
                projectList.push({
                    date: dates[i],
                    id: [i],
                    name: [`项目${i+1}`],
                })
            } else {
                projectList.push({
                    date: dates[i],
                    id: [i, i+1],
                    name: [`项目${i+1}`, `项目${i+2}`],
                })
            }
        }
        return {
            id: item,
            name: `成员${item}`,
            rowspan: 0,
            org: {
                id: item%3,
                name: `组织${item%3}`
            },
            projectList
        }
    }).sort((a, b) => a.org.id - b.org.id));
}

export default {
    sendVerificationCode,
    verifyCodeValid,
    hasMobileRegistered,
    loadTableData
}