/**
 * 从Token获取当前登录用户信息
 */
const loadUserInfo = async (token) => {
    const customHeader = token ? { Authorization: token }: {};
    return await uni.$http.GET("/sys/app/user/info", null, customHeader);
}

/**
 * 获取Token后的操作
 */
const afterToken = async (res) => {
    if (uni.$kit.checkStatus(res)) {
        // 得到token
        const token = res.data.data;
        // 获取用户登录信息，必须先请求用户基础信息，否则无法获取用户详情
        return loadUserInfo(token).then(userRes => {
            if (uni.$kit.checkStatus(userRes)) {
                userRes.data.data.token = token;
                userRes.data.data.tokenExpired = Date.getXmonthToday(1).toTimestamp(); // 一个月有效期
            }
            userRes.loginApi = true;
            return userRes;
        })
    }
    // 返回出错的结果
    res.loginApi = true;
    return res;
}

/**
 * 获取token
 * @param {String} userName
 * @param {String} password
 */
const loginByPassword = async (userName, password) => {
    /** 
     * await是内部then命令的语法糖, 后面可接如何类型的的数据，都会被转换为promise对象
     * async function getStockPriceByName(name) {
     * 		const symbol = await getStockSymbol(name);
     * 		const stockPrice = await getStockPrice(symbol);
     * 		return stockPrice;
     * }
     */
    return await uni.$http.POST("/auth/jwt/mobile/token", {
        userName,
        password
    }, uni.$http.header.noCookie).then(afterToken);
}

/**
 * 获取token
 * @param {String} mobile
 * @param {String} vcode
 */
const loginByVcode = async (mobile, vcode) => {
    return await uni.$http.POST("/auth/jwt/mobile/vcode/token", {
        userName: mobile,
        password: vcode
    }, uni.$http.header.noCookie).then(afterToken);
}

/**
 * 使用当前token重新获取token
 */
const loginByToken = async () => {
    return await uni.$http.GET("/auth/app/replace/token").then(afterToken);
}

/**
 * 获取token
 * @param {String} platform 可选值：welink
 * @param {String} code
 * @param {String} tenantId
 */
const loginByPlatform = async (platform, code, tenantId) => {
    const url = tenantId ? `/sys/app/auth/user/info/${platform}/${tenantId}/${code}` :
        `/sys/app/auth/user/info/${platform}/${code}`;
    return await uni.$http.GET(url, null, uni.$http.header.noCookie).then(res => {
        res.data.data = res.data.data.authorization;
        return res;
    }).then(afterToken);
}

export default {
    loginByPassword,
    loginByVcode,
    loginByToken,
    loginByPlatform
}
