import check from "@/common/kit/check.js"
import emptyKit from "@/common/kit/empty.js"
import logger from "@/common/kit/logger.js"

/**
 * 转换返回数据对象
 * @param {Object} res
 * @param {Object} convert
 */
function convertObj(res, convert) {
    if (check.checkStatus(res)) {
        res.data.data = convert(res.data.data);
        logger.log("convert obj: ", res.data.data);
    }
    return res;
}

/**
 * 转换返回数据列表
 * @param {Object} res
 * @param {Object} convert
 * @param {Object} filter
 */
function convertList(res, convert, filter) {
    if (check.checkStatus(res)) {
        let data = res.data.data;
        if (filter) {
            data = data.filter(filter);
        }
        res.data.data = data ? data.map(item => convert(item)) : []
        logger.log("convert list: ", res.data.data);
    }
    return res
}

/**
 * 转换返回树
 * @param {Object} res
 * @param {Object} convert
 */
function convertTree(res, convert, filter) {
    if (check.checkStatus(res)) {
        const formatData = (list = [], parent) => {
            let tree = []
            list.forEach(item => {
                if (!emptyKit.isEmpty(item.children)) {
                    item.children = formatData(item.children, { ...item, children: null });
                }

                if (!filter || filter(item, parent)) {
                    tree.push(convert({ ...item, parent }));
                }
            })
            return tree;
        }
        res.data.data = res.data.data ? formatData(res.data.data, null) : [];
        logger.log("convert tree: ", res.data.data);
    }
    return res
}

export default {
    convertObj,
    convertList,
    convertTree
}