import type from "@/common/kit/type.js";
import empty from "@/common/kit/empty.js";

/**
 * 标准化响应，便于统一处理
 * @param {Object} res
 */
function normalizeResponse(res) {
    return {
        ...res,
        statusCode: res.status,
        errMsg: res.statusText
    }
}

/**
 * 平铺树形结构
 * @param {Object} tree
 * @param {Function} filter
 */
function flatTree(tree, filter) {
    if (empty.isEmpty(tree)) {
        return []
    }

    const list = [];
    tree.forEach(item => {
        if (!filter) {
            list.push({
                ...item,
                children: null
            })
        } else if (type.isFunction(filter) && filter(item)) {
            list.push({
                ...item,
                children: null
            })
        }

        const res = flatTree(item.children, filter);
        if (!empty.isEmpty(res)) {
            list.push(...res)
        }
    })
    return list;
}

/**
 * 列表去重
 */
function unique(list, key = "id") {
    const tmpList = [];
    for (let i = 0; i < list.length; i++) {
        const item = list[i];
        if (!tmpList.find(tmpItem => tmpItem[key] == item[key])) {
            tmpList.push(item);
        }
    }
    return tmpList;
}

export default {
    normalizeResponse,
    flatTree,
    unique
}
