/**
 * 1 导出excel: https://ext.dcloud.net.cn/plugin?id=9785#detail
 * 2 
 */

function uuid() {
    return 'xxx-xxx-xxx'.replace(/[xy]/g, c => {
        var r = Math.random() * 16 | 0,
            v = c == 'x' ? r : (r & 0x3 | 0x8)
        return v.toString(16)
    })
}

/**
 * 将表格数据转换为excel文件，通过rowspan和colspan实现合并单元格
 * @param {Array} tableHeads
 * @param {Array} jsonData
 * @param {String} fileName
 * @param {String} sheetName
 */
function tableToExcel(tableHeads, jsonData, fileName, sheetName = "sh1") {
    let str = `<tr><td colspan="${tableHeads.length}" style="text-align: center;font-weight: 700;border: 0.5px solid #333333;">${fileName}</td></tr>`;
    str += "<tr>" + tableHeads.map(head => {
        if (uni.$kit.isString(head)) {
            return `<th style="text-align: center;">${head}</th>`
        } else if (uni.$kit.isObject(head)) {
            return `<th rowspan="${head.rowspan || 1}" colspan="${head.colspan || 1}" style="text-align: center;${head.style}">${head.text}</th>`
        }
    }).join("") + "</tr>"
    // 循环遍历，每行加入tr标签，每个单元格加td标签
    for (let i = 0; i < jsonData.length; i++) {
        str += '<tr>'
        for (let item in jsonData[i]) { // jsonData[i]可以是Array或Object
            // 增加\t为了不让表格显示科学计数法或者其他格式
            if (uni.$kit.isObject(jsonData[i][item])) {
                str += `<td rowspan="${jsonData[i][item].rowspan || 1}" colspan="${jsonData[i][item].colspan || 1}" style="text-align: center;${jsonData[i][item].style}">${jsonData[i][item].text + '\t'}</td>`
            } else if (uni.$kit.isString(jsonData[i][item])) {
                str += `<td style="text-align: center;">${jsonData[i][item] + '\t'}</td>`
            }
        }
        str += '</tr>'
    }
    toExcel(str, fileName, sheetName);
}

/**
 * 将表格html字符串转换为excel文件
 * @param {String} str
 * @param {String} fileName
 * @param {String} sheetName
 */
function toExcel(str, fileName = "excel", sheetName = "sh1") {
    // 下载的表格模板数据
    let template = `<html xmlns:o="urn:schemas-microsoft-com:office:office" 
        xmlns:x="urn:schemas-microsoft-com:office:excel" 
        xmlns="http://www.w3.org/TR/REC-html40">
        <head><!--[if gte mso 9]><xml encoding="UTF-8"><x:ExcelWorkbook><x:ExcelWorksheets><x:ExcelWorksheet>
        <x:Name>${sheetName}</x:Name>
        <x:WorksheetOptions><x:DisplayGridlines/></x:WorksheetOptions></x:ExcelWorksheet>
        </x:ExcelWorksheets></x:ExcelWorkbook></xml><![endif]-->
        </head><body><table>${str}</table></body></html>`
    // 下载模板
    // #ifdef APP-PLUS
    appExportFile(template, fileName)
    // #endif
    // 下载模板
    // #ifdef MP-WEIXIN
    wxExportFile(template, fileName)
    // #endif
}

// #ifdef APP-PLUS
// 导出文件到手机 fileData:要写入到文件的数据，返回参数为文档路径
function appExportFile(fileData, documentName = 'excel') {
    // PUBLIC_DOCUMENTS: 程序公用文档目录常量
    plus.io.requestFileSystem(plus.io.PUBLIC_DOCUMENTS, fs => {
        let rootObj = fs.root,
            fullPath = rootObj.fullPath
        // 创建同名文件夹
        rootObj.getDirectory(documentName, {
            create: true
        }, dirEntry => {
            // 创建文件,防止重名
            let fileName = documentName + uuid() + '.xlsx'
            dirEntry.getFile(fileName, {
                create: true
            }, fileEntry => {
                fileEntry.createWriter(writer => {
                    writer.onwritestart = res => console.log('正在导出')
                    //  /storage/emulated/0指的就是系统路径
                    let pathStr = fullPath.replace('/storage/emulated/0', '')
                    // 成功导出数据
                    writer.onwrite = res => {
                        // 文件路径
                        let filePath = res.target.fileName
                        uni.hideLoading()
                        setTimeout(() => {
                            console.log(`成功导出，文件位置：${filePath}`)
                            uni.openDocument({
                                filePath,
                                success: res => console.log('打开文档成功'),
                                fail: console.log
                            })
                        }, 500)
                    }
                    // 写入内容
                    writer.write(fileData)
                }, err => console.log(err.message))
            })
        })
    })
}
// #endif

// #ifdef MP-WEIXIN
function wxExportFile(template, documentName = 'excel') {
    const fs = wx.getFileSystemManager()
    // 创建文件名字, 防止重名
    let filePath = wx.env.USER_DATA_PATH + '/' + (documentName + uuid()) + '.xls'
    fs.writeFile({
        filePath,
        data: template,
        encoding: 'utf8',
        success: res => {
            wx.openDocument({
                filePath,
                success: res => console.log('打开文档成功。文件位置: ', filePath),
                fail: console.log
            })
        },
        fail: console.info
    })
}
// #endif

export default {
    tableToExcel,
    toExcel
}
