import App from './App'

/** -------------------------安装插件------------------------------
 * 说明：插件通常用来为Vue(VUE2)或app(VUE3)添加全局功能，插件的功能范围没有严格的限制：
 * （1）添加全局方法或者 property。如：vue-custom-element
 * （2）添加全局资源：指令/过滤器/过渡等。如：vue-touch
 * （3）通过全局混入来添加一些组件选项。如： vue-router
 * （4）添加 Vue 实例方法，通过把它们添加到 Vue.prototype 上实现
 * （5）一个库，提供自己的 API，同时提供上面提到的一个或多个功能，如： vue-router
 * 注意：通过全局方法Vue.use(VUE2)或app.use(VUE3)使用插件，它需要在你VUE2中调用new Vue()启动应用之前完成
 */
// vuex状态管理插件
import store from '@/store'
// 国际化插件，注意：页面中设置语言后需要调用 this.$i18n.locale = 'zh-Hans' 后生效
import i18n from './local'
// http请求
import request from "@/common/request/http.install.js"

/** -------------------------直接混入-------------------------------
 * 说明：混入 (mixin) 提供了一种非常灵活的方式，来分发 Vue 组件中的可复用功能。一个混入对象可以包含任意组件选项。
 * 当组件使用混入对象时，所有混入对象的选项将被“混合”进入该组件本身的选项。
 * 注意：当组件和混入对象含有同名选项时，这些选项将以恰当的方式进行“合并”
 * （1）数据对象在内部会进行递归合并，并在发生冲突时以组件数据优先
 * （2）同名钩子函数将合并为一个数组，因此都将被调用。另外，混入对象的钩子将在组件自身钩子之前调用
 *     其中包括：周期钩子、watch方法等
 * （3）methods、components 和 directives，将被合并为同一个对象。两个对象键名冲突时，取组件对象的键值对
 * （4）Nvue不支持全局Mixin，Prototype
 */

// 引入uView提供的对vuex的简写法文件，vuex插件也在其中安装
import vuexMixin from '@/store/store.install.js'
// 安装自定义混入框架，通过uni.$mixer使用
import mixer from '@/common/mixin/index.install.js'
// 工具方法
import kit from "@/common/kit/index.install.js"
import config from "@/config.js";

// #ifdef VUE2
import Vue from 'vue'
import uView from '@/uni_modules/uview-ui'
Vue.use(vuexMixin)
Vue.use(mixer)
Vue.use(kit)
Vue.use(request)
Vue.use(uView)

Vue.prototype.$config = config

// VUE3不再需要下面行代码
Vue.config.productionTip = false
App.mpType = 'app'

const app = new Vue({
    i18n,
    store,
    ...App
})
app.$mount()
// #endif

// #ifdef VUE3
import { createSSRApp } from 'vue'
export function createApp() {
  const app = createSSRApp(App)
  app.use(i18n)
  app.use(store)
  app.use(vuexMixin)
  app.use(mixer)
  app.use(kit)
  app.use(request)
  app.config.globalProperties.$config = config
  return { app }
}
// #endif