/**
 * 说明：Action 类似于 mutation，不同在于：
 * （1）Action 提交的是 mutation，而不是直接变更状态。
 * （2）Action 可以包含任意异步操作。
 * 原理：说明：Action 函数接受一个与 store 实例具有相同方法和属性的 context 对象，因此你可以调用 context.commit 提交一个 mutation
 * 使用方法：
 * （1）action通过this.$store.dispatch('xxx')分发action
 * （2）mapActions辅助函数将组件的methods映射为this.$store.dispatch调用
 */

export default {
    /**
     * 说明：接收两个参数：({ dispatch, commit, state, rootState, getters, rootGetters } = context, params) => {}
     * vuex可以模块化状态机，因此这里的state和rootState分别代表当前子模块和根模块的状态
     */
    $vuexAction({ rootState }, params) {
        let nameArr = params.name.split('.');
        if (nameArr.length >= 2) {
            let obj = rootState[nameArr[0]];
            for (let i = 1; i < nameArr.length - 1; i++) {
                obj = obj[nameArr[i]];
            }
            obj[nameArr[nameArr.length - 1]] = params.value;
        } else {
            rootState[params.name] = params.value;
        }
    }
}
