import {
    mapState,
    mapGetters
} from 'vuex'
import store from "@/store"

// 尝试将用户在根目录中的store/index.js的vuex的state变量，全部加载到全局变量中
let $uStoreKeys = [];
let $uGettersKeys = [];
let $uGettersAliasKeys = [];
try {
    $uStoreKeys = store.state ? Object.keys(store.state) : [];
    $uGettersKeys = store.getters ? Object.keys(store.getters) : [];
    $uGettersAliasKeys = $uGettersKeys.map(getter => getter.replace(/\//g, "_")); // 注：将模块化的getters用 "_"替换 "/" 拼接
} catch (e) {

}

const $uMapKeys = {}
for (let i = 0; i < $uGettersAliasKeys.length; i++) {
    $uMapKeys[$uGettersAliasKeys[i]] = $uGettersKeys[i]
}

const mixinStore = {
    created() {
        // 将vuex方法挂在到$u中
        // 使用方法为：如果要修改vuex的state中的user.name变量为"史诗" => this.$u.vuex('user.name', '史诗')
        // 如果要修改vuex的state的version变量为1.0.1 => this.$u.vuex('version', '1.0.1')
        this.$u.vuex = (name, value) => {
            this.$store.commit('$vuexMutation', {
                name,
                value
            })
        }

        /**
         * 清除所有数据
         */
        this.$u.clearVuex = () => {
            this.$store.commit('$uClearVuex')
        }
    },

    computed: {
        // 将vuex的state中的所有变量，解构到全局混入的mixin中
        // ...mapState($uStoreKeys),
        // ...mapGetters($uMapKeys)
    }
}

// 用来挂载到uni对象，可通过uni.$u对象操作vuex变量
const $u = {
    /**
     * 保存变量
     */
    vuex: function (name, value) {
        store.commit('$vuexMutation', {
            name,
            value
        })
    },
    
    /**
     * 清除所有数据
     */
    clearVuex: function () {
        store.commit('$uClearVuex')
    }
}

// 挂载到uni.$u
uni.$u = $u

/**
 * 使用app.use (Vue.use)安装插件，必须提供install方法
 */
export default {
    install: app => {
        // #ifdef VUE2
        if (!app.prototype.$u) {
            app.prototype.$u = {}
        }
        // #endif
        // #ifdef VUE3
        app.config.globalProperties.$u = {};
        // #endif
        app.mixin(mixinStore);
    }
}
